@@ -35,7 +35,6 @@ menuitem_cb (gpointer             callback_data,
 static GtkItemFactoryEntry menu_items[] =
 {
   { "/_File",            NULL,         0,                     0, "<Branch>" },
-  { "/File/tearoff1",    NULL,         menuitem_cb,       0, "<Tearoff>" },
   { "/File/_New",        "<control>N", menuitem_cb,       0, "<StockItem>", GTK_STOCK_NEW },
   { "/File/_Open",       "<control>O", menuitem_cb,       0, "<StockItem>", GTK_STOCK_OPEN },
   { "/File/_Save",       "<control>S", menuitem_cb,       0, "<StockItem>", GTK_STOCK_SAVE },
@@ -217,7 +216,7 @@ do_appwindow (void)
       item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", accel_group);
 
       /* Set up item factory to go away with the window */
-      gtk_object_ref (GTK_OBJECT (item_factory));
+      g_object_ref (item_factory);
       gtk_object_sink (GTK_OBJECT (item_factory));
       g_object_set_data_full (G_OBJECT (window),
                               "<main>",
@@ -229,7 +228,7 @@ do_appwindow (void)
                                      menu_items, window);
 
       gtk_table_attach (GTK_TABLE (table),
-                        gtk_item_factory_get_widget (item_factory, "<main>"),
+			gtk_item_factory_get_widget (item_factory, "<main>"),
                         /* X direction */          /* Y direction */
                         0, 1,                      0, 1,
                         GTK_EXPAND | GTK_FILL,     0,
