@@ -371,8 +371,14 @@ GdkWindowState gdk_window_get_state (GdkWindow *window);
 gboolean gdk_window_set_static_gravities (GdkWindow *window,
 					  gboolean   use_static);   
 
+/* Functions to create/lookup windows from their native equivalents */ 
+GdkWindow*    gdk_window_foreign_new (GdkNativeWindow anid);
+GdkWindow*    gdk_window_lookup      (GdkNativeWindow anid);
+
+
 /* GdkWindow */
 
+#ifndef GDK_DISABLE_DEPRECATED
 void	      gdk_window_set_hints	 (GdkWindow	  *window,
 					  gint		   x,
 					  gint		   y,
@@ -381,6 +387,7 @@ void	      gdk_window_set_hints	 (GdkWindow	  *window,
 					  gint		   max_width,
 					  gint		   max_height,
 					  gint		   flags);
+#endif
 void          gdk_window_set_type_hint   (GdkWindow       *window,
 					  GdkWindowTypeHint hint);
 void          gdk_window_set_modal_hint  (GdkWindow       *window,
