@@ -527,7 +527,7 @@ gdk_colormap_alloc1 (GdkColormap *colormap,
 
   for (i = 0; i<colormap->size; i++)
     {
-      if (!(private->info[i].flags && GDK_COLOR_WRITEABLE) &&
+      if (!(private->info[i].flags & GDK_COLOR_WRITEABLE) &&
 	  (ret->red == colormap->colors[i].red) &&
 	  (ret->green == colormap->colors[i].green) &&
 	  (ret->blue == colormap->colors[i].blue))
@@ -605,7 +605,7 @@ gdk_colormap_alloc_colors_shared (GdkColormap *colormap,
 
       for (i = 0; i < colormap->size; i++)
 	available[i] = ((private->info[i].ref_count == 0) ||
-			!(private->info[i].flags && GDK_COLOR_WRITEABLE));
+			!(private->info[i].flags & GDK_COLOR_WRITEABLE));
       
       while (nremaining > 0)
 	{
