@@ -36,6 +36,8 @@
 #include "gdkdrawable-win32.h"
 #include "gdkwindow-win32.h"
 
+#define SIZE_LIMIT 32000
+
 typedef struct _GdkWindowQueueItem GdkWindowQueueItem;
 typedef struct _GdkWindowParentPos GdkWindowParentPos;
 
@@ -148,6 +150,10 @@ gdk_window_scroll (GdkWindow *window,
   if (GDK_WINDOW_DESTROYED (window))
     return;
 
+  /* Move the current invalid region */
+  if (obj->update_area)
+    gdk_region_offset (obj->update_area, dx, dy);
+
   /* We can guffaw scroll if we are a child window, and the parent
    * does not extend beyond our edges.
    */
@@ -163,6 +169,47 @@ gdk_window_scroll (GdkWindow *window,
 
   if (!obj->children || !can_guffaw_scroll)
     {
+#if 0 /* this may be the better way to do it, though there seems to 
+         be an offset error in it */
+      GList *tmp_list = obj->children;
+      int num_windows = g_list_length (tmp_list) + 1;
+      HDWP hWinPosInfo = BeginDeferWindowPos (num_windows);
+
+      GDK_NOTE (MISC, 
+                g_print ("gdk_window_scroll (%x, %d, %d), %d childs\n",
+                         GDK_WINDOW_HWND (window), dx, dy, num_windows -1));
+      if (!hWinPosInfo)
+        WIN32_API_FAILED ("BeginDeferWindowPos");
+      else
+        {
+
+          hWinPosInfo = 
+            DeferWindowPos (hWinPosInfo, /* may change on return */
+                            GDK_WINDOW_HWND (window),
+                            NULL, /* placement-order handle, ignored */
+                            obj->x + dx, obj->y + dy, /* new pos */
+                            30, 20,  /* new size, ignored */
+                            SWP_NOSIZE | SWP_NOZORDER);
+
+          while (tmp_list)
+	    {
+	      GdkWindowObject * child = GDK_WINDOW_OBJECT (tmp_list->data);
+	  
+              hWinPosInfo = 
+                DeferWindowPos (hWinPosInfo, /* may change on return */
+                                GDK_WINDOW_HWND (child),
+                                NULL, /* placement-order handle, ignored */
+                                child->x + dx, child->y + dy, /* new pos */
+                                30, 20,  /* new size, ignored */
+                                SWP_NOSIZE | SWP_NOZORDER);
+	  
+	      tmp_list = tmp_list->next;
+	    }
+
+          if (!EndDeferWindowPos (hWinPosInfo))
+	    WIN32_API_FAILED ("EndDeferWindowPos");
+       }
+#else
       /* Use ScrollWindowEx, then move any children later
        */
       GList *tmp_list;
@@ -203,6 +250,7 @@ gdk_window_scroll (GdkWindow *window,
 	  
 	  tmp_list = tmp_list->next;
 	}
+#endif
     }
   else
     {
@@ -295,10 +343,10 @@ _gdk_window_move_resize_child (GdkWindow *window,
 	  new_y1 = impl->position_info.y + new_info.height + d_yoffset;
 	}
       
-      if (!MoveWindow (GDK_WINDOW_HWND (window),
+      if (!SetWindowPos (GDK_WINDOW_HWND (window), NULL,
 		       new_x0, new_y0, new_x1 - new_x0, new_y1 - new_y0,
-		       FALSE))
-	  WIN32_API_FAILED ("MoveWindow");
+		       SWP_NOACTIVATE | SWP_NOZORDER | SWP_NOREDRAW))
+	  WIN32_API_FAILED ("SetWindowPos");
       
       tmp_list = obj->children;
       while (tmp_list)
@@ -307,26 +355,30 @@ _gdk_window_move_resize_child (GdkWindow *window,
 	  tmp_list = tmp_list->next;
 	}
 
-      GetClientRect (GDK_WINDOW_HWND (window), &rect);
-
-      if (!MoveWindow (GDK_WINDOW_HWND (window),
-		       new_x0 + dx, new_y0 + dy,
-		       rect.right - rect.left, rect.bottom - rect.top,
-		       FALSE))
-	  WIN32_API_FAILED ("MoveWindow");
+      if (!SetWindowPos (GDK_WINDOW_HWND (window), NULL,
+		       new_x0 + dx, new_y0 + dy, 0, 0,
+		       SWP_NOACTIVATE | SWP_NOZORDER | SWP_NOSIZE | SWP_NOREDRAW))
+	  WIN32_API_FAILED ("SetWindowPos");
       
       if (d_xoffset > 0 || d_yoffset > 0)
 	gdk_window_queue_translation (window, MAX (d_xoffset, 0), MAX (d_yoffset, 0));
-      
-      if (!MoveWindow (GDK_WINDOW_HWND (window),
+
+      /* FIXME: 
+       */
+      if (new_info.x + new_info.width > SIZE_LIMIT)
+        new_info.width = SIZE_LIMIT - new_info.x;
+      if (new_info.y + new_info.height > SIZE_LIMIT)
+        new_info.height = SIZE_LIMIT - new_info.y;
+
+      if (!SetWindowPos (GDK_WINDOW_HWND (window), NULL,
 		       new_info.x, new_info.y, new_info.width, new_info.height,
-		       FALSE))
-	  WIN32_API_FAILED ("MoveWindow");
+		       SWP_NOACTIVATE | SWP_NOZORDER | SWP_NOREDRAW))
+	  WIN32_API_FAILED ("SetWindowPos");
       
       if (impl->position_info.no_bg)
 	gdk_window_tmp_reset_bg (window);
 
-      if (!impl->position_info.mapped && new_info.mapped && obj->mapped)
+      if (!impl->position_info.mapped && new_info.mapped && GDK_WINDOW_IS_MAPPED (obj))
 	ShowWindow (GDK_WINDOW_HWND (window), SW_SHOWNA);
       
       impl->position_info = new_info;
@@ -353,28 +405,11 @@ _gdk_window_move_resize_child (GdkWindow *window,
 	  tmp_list = tmp_list->next;
 	}
 
-      /*
-       * HB: Passing TRUE(=Redraw) to MoveWindow here fixes some
-       * redraw problems with (e.g. testgtk main buttons)
-       * scrolling. AFAIK the non flicker optimization would
-       * be done by the GDI anyway, if the window is SW_HIDE.
-       */
-      if (is_resize)
-	{
-	  if (!MoveWindow (GDK_WINDOW_HWND (window),
+      if (!SetWindowPos (GDK_WINDOW_HWND (window), NULL,
 			   new_info.x, new_info.y, new_info.width, new_info.height,
-			   TRUE /*FALSE*/))
-	    WIN32_API_FAILED ("MoveWindow");
-	}
-      else
-	{
-	  GetClientRect (GDK_WINDOW_HWND (window), &rect);
-	  if (!MoveWindow (GDK_WINDOW_HWND (window),
-			   new_info.x, new_info.y,
-			   rect.right - rect.left, rect.bottom - rect.top,
-			   TRUE /*FALSE*/))
-	    WIN32_API_FAILED ("MoveWindow");
-	}
+			   SWP_NOACTIVATE | SWP_NOZORDER |
+			   (is_resize ? 0 : SWP_NOSIZE)))
+	  WIN32_API_FAILED ("SetWindowPos");
 
       tmp_list = obj->children;
       while (tmp_list)
@@ -386,7 +421,7 @@ _gdk_window_move_resize_child (GdkWindow *window,
       if (impl->position_info.no_bg)
 	gdk_window_tmp_reset_bg (window);
 
-      if (!impl->position_info.mapped && new_info.mapped && obj->mapped)
+      if (!impl->position_info.mapped && new_info.mapped && GDK_WINDOW_IS_MAPPED (obj))
 	ShowWindow (GDK_WINDOW_HWND (window), SW_SHOWNA);
 
       impl->position_info = new_info;
@@ -408,7 +443,7 @@ gdk_window_compute_position (GdkWindowImplWin32   *window,
 
   info->big = FALSE;
   
-  if (window->width <= 32768)
+  if (window->width <= SIZE_LIMIT)
     {
       info->width = window->width;
       info->x = parent_pos->x + wrapper->x - parent_pos->win32_x;
@@ -416,19 +451,19 @@ gdk_window_compute_position (GdkWindowImplWin32   *window,
   else
     {
       info->big = TRUE;
-      info->width = 32768;
-      if (parent_pos->x + wrapper->x < -16384)
+      info->width = SIZE_LIMIT;
+      if (parent_pos->x + wrapper->x < -(SIZE_LIMIT/2))
 	{
-	  if (parent_pos->x + wrapper->x + window->width < 16384)
-	    info->x = parent_pos->x + wrapper->x + window->width - 32768 - parent_pos->win32_x;
+	  if (parent_pos->x + wrapper->x + window->width < (SIZE_LIMIT/2))
+	    info->x = parent_pos->x + wrapper->x + window->width - SIZE_LIMIT - parent_pos->win32_x;
 	  else
-	    info->x = -16384 - parent_pos->win32_y;
+	    info->x = -(SIZE_LIMIT/2) - parent_pos->win32_y;
 	}
       else
 	info->x = parent_pos->x + wrapper->x - parent_pos->win32_x;
     }
 
-  if (window->height <= 32768)
+  if (window->height <= SIZE_LIMIT)
     {
       info->height = window->height;
       info->y = parent_pos->y + wrapper->y - parent_pos->win32_y;
@@ -436,13 +471,13 @@ gdk_window_compute_position (GdkWindowImplWin32   *window,
   else
     {
       info->big = TRUE;
-      info->height = 32768;
-      if (parent_pos->y + wrapper->y < -16384)
+      info->height = SIZE_LIMIT;
+      if (parent_pos->y + wrapper->y < -(SIZE_LIMIT/2))
 	{
-	  if (parent_pos->y + wrapper->y + window->height < 16384)
-	    info->y = parent_pos->y + wrapper->y + window->height - 32768 - parent_pos->win32_y;
+	  if (parent_pos->y + wrapper->y + window->height < (SIZE_LIMIT/2))
+	    info->y = parent_pos->y + wrapper->y + window->height - SIZE_LIMIT - parent_pos->win32_y;
 	  else
-	    info->y = -16384 - parent_pos->win32_y;
+	    info->y = -(SIZE_LIMIT/2) - parent_pos->win32_y;
 	}
       else
 	info->y = parent_pos->y + wrapper->y - parent_pos->win32_y;
@@ -610,10 +645,10 @@ gdk_window_premove (GdkWindow          *window,
 	  new_y1 = impl->position_info.y + new_info.height + d_yoffset;
 	}
 
-      if (!MoveWindow (GDK_WINDOW_HWND (window),
+      if (!SetWindowPos (GDK_WINDOW_HWND (window), NULL,
 		       new_x0, new_y0, new_x1 - new_x0, new_y1 - new_y0,
-		       FALSE))
-	WIN32_API_FAILED ("MoveWindow");
+		       SWP_NOREDRAW | SWP_NOZORDER | SWP_NOACTIVATE))
+	WIN32_API_FAILED ("SetWindowPos");
     }
 
   tmp_list = obj->children;
@@ -654,13 +689,13 @@ gdk_window_postmove (GdkWindow          *window,
       if (d_xoffset > 0 || d_yoffset > 0)
 	gdk_window_queue_translation (window, MAX (d_xoffset, 0), MAX (d_yoffset, 0));
 	
-      if (!MoveWindow (GDK_WINDOW_HWND (window),
+      if (!SetWindowPos (GDK_WINDOW_HWND (window), NULL,
 		       new_info.x, new_info.y, new_info.width, new_info.height,
-		       FALSE))
-	WIN32_API_FAILED ("MoveWindow");
+		       SWP_NOREDRAW | SWP_NOZORDER | SWP_NOACTIVATE))
+	WIN32_API_FAILED ("SetWindowPos");
     }
 
-  if (!impl->position_info.mapped && new_info.mapped && obj->mapped)
+  if (!impl->position_info.mapped && new_info.mapped && GDK_WINDOW_IS_MAPPED (obj))
     ShowWindow (GDK_WINDOW_HWND (window), SW_SHOWNA);
 
   if (impl->position_info.no_bg)
@@ -701,7 +736,7 @@ gboolean
 _gdk_windowing_window_queue_antiexpose (GdkWindow *window,
 					GdkRegion *area)
 {
-#if 0
+#if 1
   GdkWindowQueueItem *item = g_new (GdkWindowQueueItem, 1);
 
   item->window = window;
@@ -808,8 +843,12 @@ gdk_window_tmp_unset_bg (GdkWindow *window)
 
   impl->position_info.no_bg = TRUE;
 
+  /*
+   * The X version sets background = None to avoid updateing for a moment.
+   * Not sure if this could really emulate it.
+   */
   if (obj->bg_pixmap != GDK_NO_BG)
-    /* ??? */;
+    /* handled in WM_ERASEBKGRND proceesing */;
 }
 
 static void
