@@ -39,31 +39,31 @@ static GdkDeviceAxis gdk_input_core_axes[] = {
   { GDK_AXIS_Y, 0, 0 }
 };
 
-GdkDevice *gdk_core_pointer = NULL;
+GdkDevice *_gdk_core_pointer = NULL;
  
 /* Global variables  */
 
 /* information about network port and host for gxid daemon */
-gchar            *gdk_input_gxid_host;
-gint              gdk_input_gxid_port;
-gint              gdk_input_ignore_core;
+gchar            *_gdk_input_gxid_host;
+gint              _gdk_input_gxid_port;
+gint              _gdk_input_ignore_core;
 
-GList            *gdk_input_devices;
-GList            *gdk_input_windows;
+GList            *_gdk_input_devices;
+GList            *_gdk_input_windows;
 
 void
 _gdk_init_input_core (void)
 {
-  gdk_core_pointer = g_object_new (GDK_TYPE_DEVICE, NULL);
+  _gdk_core_pointer = g_object_new (GDK_TYPE_DEVICE, NULL);
   
-  gdk_core_pointer->name = "Core Pointer";
-  gdk_core_pointer->source = GDK_SOURCE_MOUSE;
-  gdk_core_pointer->mode = GDK_MODE_SCREEN;
-  gdk_core_pointer->has_cursor = TRUE;
-  gdk_core_pointer->num_axes = 2;
-  gdk_core_pointer->axes = gdk_input_core_axes;
-  gdk_core_pointer->num_keys = 0;
-  gdk_core_pointer->keys = NULL;
+  _gdk_core_pointer->name = "Core Pointer";
+  _gdk_core_pointer->source = GDK_SOURCE_MOUSE;
+  _gdk_core_pointer->mode = GDK_MODE_SCREEN;
+  _gdk_core_pointer->has_cursor = TRUE;
+  _gdk_core_pointer->num_axes = 2;
+  _gdk_core_pointer->axes = gdk_input_core_axes;
+  _gdk_core_pointer->num_keys = 0;
+  _gdk_core_pointer->keys = NULL;
 }
 
 GType
@@ -97,7 +97,7 @@ gdk_device_get_type (void)
 GList *
 gdk_devices_list (void)
 {
-  return gdk_input_devices;
+  return _gdk_input_devices;
 }
 
 void
@@ -233,7 +233,7 @@ gdk_input_window_find(GdkWindow *window)
 {
   GList *tmp_list;
 
-  for (tmp_list=gdk_input_windows; tmp_list; tmp_list=tmp_list->next)
+  for (tmp_list=_gdk_input_windows; tmp_list; tmp_list=tmp_list->next)
     if (((GdkInputWindow *)(tmp_list->data))->window == window)
       return (GdkInputWindow *)(tmp_list->data);
 
@@ -275,7 +275,7 @@ gdk_input_set_extension_events (GdkWindow *window, gint mask,
       iw->num_obscuring = 0;
       iw->grabbed = FALSE;
 
-      gdk_input_windows = g_list_append(gdk_input_windows,iw);
+      _gdk_input_windows = g_list_append(_gdk_input_windows,iw);
       window_private->extension_events = mask;
 
       /* Add enter window events to the event mask */
@@ -289,14 +289,14 @@ gdk_input_set_extension_events (GdkWindow *window, gint mask,
       iw = gdk_input_window_find (window);
       if (iw)
 	{
-	  gdk_input_windows = g_list_remove(gdk_input_windows,iw);
+	  _gdk_input_windows = g_list_remove(_gdk_input_windows,iw);
 	  g_free(iw);
 	}
 
       window_private->extension_events = 0;
     }
 
-  for (tmp_list = gdk_input_devices; tmp_list; tmp_list = tmp_list->next)
+  for (tmp_list = _gdk_input_devices; tmp_list; tmp_list = tmp_list->next)
     {
       GdkDevicePrivate *gdkdev = tmp_list->data;
 
@@ -319,17 +319,17 @@ gdk_input_window_destroy (GdkWindow *window)
   input_window = gdk_input_window_find (window);
   g_return_if_fail (input_window != NULL);
 
-  gdk_input_windows = g_list_remove (gdk_input_windows,input_window);
+  _gdk_input_windows = g_list_remove (_gdk_input_windows,input_window);
   g_free(input_window);
 }
 
 void
-gdk_input_exit (void)
+_gdk_input_exit (void)
 {
   GList *tmp_list;
   GdkDevicePrivate *gdkdev;
 
-  for (tmp_list = gdk_input_devices; tmp_list; tmp_list = tmp_list->next)
+  for (tmp_list = _gdk_input_devices; tmp_list; tmp_list = tmp_list->next)
     {
       gdkdev = (GdkDevicePrivate *)(tmp_list->data);
       if (!GDK_IS_CORE (gdkdev))
@@ -346,12 +346,12 @@ gdk_input_exit (void)
 	}
     }
 
-  g_list_free(gdk_input_devices);
+  g_list_free(_gdk_input_devices);
 
-  for (tmp_list = gdk_input_windows; tmp_list; tmp_list = tmp_list->next)
+  for (tmp_list = _gdk_input_windows; tmp_list; tmp_list = tmp_list->next)
     g_free(tmp_list->data);
 
-  g_list_free(gdk_input_windows);
+  g_list_free(_gdk_input_windows);
 }
 
 /**
