@@ -150,6 +150,8 @@ void     _gdk_keymap_state_changed         (void);
 
 GC _gdk_x11_gc_flush (GdkGC *gc);
 
+void _gdk_x11_initialize_locale (void);
+
 extern GdkDrawableClass  _gdk_x11_drawable_class;
 extern Window		 _gdk_root_window;
 extern gboolean	         _gdk_use_xshm;
@@ -157,7 +159,7 @@ extern Atom		 _gdk_wm_window_protocols[];
 extern const int         _gdk_nenvent_masks;
 extern const int         _gdk_event_mask_table[];
 extern gint		 _gdk_screen;
-extern Atom		 _gdk_selection_property;
+extern GdkAtom		 _gdk_selection_property;
 extern gchar		*_gdk_display_name;
 
 extern Window		 _gdk_leader_window;
