@@ -61,7 +61,7 @@ gtk_adjustment_get_type (void)
         (GtkClassInitFunc) NULL,
       };
 
-      adjustment_type = gtk_type_unique (GTK_TYPE_DATA, &adjustment_info);
+      adjustment_type = gtk_type_unique (GTK_TYPE_OBJECT, &adjustment_info);
     }
 
   return adjustment_type;
@@ -74,6 +74,9 @@ gtk_adjustment_class_init (GtkAdjustmentClass *class)
 
   object_class = (GtkObjectClass*) class;
 
+  class->changed = NULL;
+  class->value_changed = NULL;
+
   adjustment_signals[CHANGED] =
     gtk_signal_new ("changed",
                     GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
@@ -88,11 +91,6 @@ gtk_adjustment_class_init (GtkAdjustmentClass *class)
                     GTK_SIGNAL_OFFSET (GtkAdjustmentClass, value_changed),
                     gtk_marshal_VOID__VOID,
 		    GTK_TYPE_NONE, 0);
-
-  gtk_object_class_add_signals (object_class, adjustment_signals, LAST_SIGNAL);
-
-  class->changed = NULL;
-  class->value_changed = NULL;
 }
 
 static void
@@ -107,12 +105,12 @@ gtk_adjustment_init (GtkAdjustment *adjustment)
 }
 
 GtkObject*
-gtk_adjustment_new (gfloat value,
-		    gfloat lower,
-		    gfloat upper,
-		    gfloat step_increment,
-		    gfloat page_increment,
-		    gfloat page_size)
+gtk_adjustment_new (gdouble value,
+		    gdouble lower,
+		    gdouble upper,
+		    gdouble step_increment,
+		    gdouble page_increment,
+		    gdouble page_size)
 {
   GtkAdjustment *adjustment;
 
@@ -128,11 +126,27 @@ gtk_adjustment_new (gfloat value,
   return GTK_OBJECT (adjustment);
 }
 
+/**
+ * gtk_adjustment_get_value:
+ * @adjustment: a #GtkAdjustment
+ *
+ * Gets the current value of the adjustment. See
+ * gtk_adjustment_set_value ().
+ *
+ * Return value: The current value of the adjustment.
+ **/
+gdouble
+gtk_adjustment_get_value (GtkAdjustment *adjustment)
+{
+  g_return_val_if_fail (GTK_IS_ADJUSTMENT (adjustment), 0.);
+
+  return adjustment->value;
+}
+
 void
 gtk_adjustment_set_value (GtkAdjustment        *adjustment,
-			  gfloat                value)
+			  gdouble               value)
 {
-  g_return_if_fail (adjustment != NULL);
   g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
 
   value = CLAMP (value, adjustment->lower, adjustment->upper);
@@ -148,7 +162,6 @@ gtk_adjustment_set_value (GtkAdjustment        *adjustment,
 void
 gtk_adjustment_changed (GtkAdjustment        *adjustment)
 {
-  g_return_if_fail (adjustment != NULL);
   g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
 
   gtk_signal_emit (GTK_OBJECT (adjustment), adjustment_signals[CHANGED]);
@@ -157,7 +170,6 @@ gtk_adjustment_changed (GtkAdjustment        *adjustment)
 void
 gtk_adjustment_value_changed (GtkAdjustment        *adjustment)
 {
-  g_return_if_fail (adjustment != NULL);
   g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
 
   gtk_signal_emit (GTK_OBJECT (adjustment), adjustment_signals[VALUE_CHANGED]);
@@ -165,12 +177,11 @@ gtk_adjustment_value_changed (GtkAdjustment        *adjustment)
 
 void
 gtk_adjustment_clamp_page (GtkAdjustment *adjustment,
-			   gfloat         lower,
-			   gfloat         upper)
+			   gdouble        lower,
+			   gdouble        upper)
 {
-  gint need_emission;
+  gboolean need_emission;
 
-  g_return_if_fail (adjustment != NULL);
   g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
 
   lower = CLAMP (lower, adjustment->lower, adjustment->upper);
