@@ -100,7 +100,7 @@ gtk_alignment_class_init (GtkAlignmentClass *class)
                                    PROP_XALIGN,
                                    g_param_spec_float("xalign",
                                                       _("Horizontal alignment"),
-                                                      _("Value between 0.0 and 1.0 to indicate X alignment"),
+                                                      _("Horizontal position of child in available space. 0.0 is left aligned, 1.0 is right aligned"),
                                                       0.0,
                                                       1.0,
                                                       0.5,
@@ -110,7 +110,7 @@ gtk_alignment_class_init (GtkAlignmentClass *class)
                                    PROP_YALIGN,
                                    g_param_spec_float("yalign",
                                                       _("Vertical alignment"),
-                                                      _("Value between 0.0 and 1.0 to indicate Y alignment"),
+                                                      _("Vertical position of child in available space. 0.0 is top aligned, 1.0 is bottom aligned"),
                                                       0.0,
                                                       1.0,
 						      0.5,
@@ -119,7 +119,7 @@ gtk_alignment_class_init (GtkAlignmentClass *class)
                                    PROP_XSCALE,
                                    g_param_spec_float("xscale",
                                                       _("Horizontal scale"),
-                                                      _("Value between 0.0 and 1.0 to indicate X scale"),
+                                                      _("If available horizontal space is bigger than needed for the child, how much of it to use for the child. 0.0 means none, 1.0 means all"),
                                                       0.0,
                                                       1.0,
                                                       1.0,
@@ -128,7 +128,7 @@ gtk_alignment_class_init (GtkAlignmentClass *class)
                                    PROP_YSCALE,
                                    g_param_spec_float("yscale",
                                                       _("Vertical scale"),
-                                                      _("Value between 0.0 and 1.0 to indicate Y scale"),
+                                                      _("If available vertical space is bigger than needed for the child, how much of it to use for the child. 0.0 means none, 1.0 means all"),
                                                       0.0,
                                                       1.0,
                                                       1.0,
