@@ -88,10 +88,10 @@ static GQuark clipboards_owned_key_id = 0;
 /**
  * gtk_clipboard_get:
  * @selection: a #GdkAtom which identifies the clipboard
- *             to use. A value of GDK_NONE here is the
+ *             to use. A value of %GDK_NONE here is the
  *             same as gdk_atom_intern ("CLIPBOARD", FALSE),
  *             and provides the default clipboard. Another
- *             common value is GDK_SELECTION_PRIMARY, which
+ *             common value is %GDK_SELECTION_PRIMARY, which
  *             identifies the primary X selection. 
  * 
  * Returns the clipboard object for the given selection.
@@ -99,7 +99,7 @@ static GQuark clipboards_owned_key_id = 0;
  * Return value: the appropriate clipboard object. If no
  *             clipboard already exists, a new one will
  *             be created. Once a clipboard object has
- *             been created, it is persistant for all time.
+ *             been created, it is persistent for all time.
  **/
 GtkClipboard *
 gtk_clipboard_get (GdkAtom selection)
@@ -359,10 +359,10 @@ gtk_clipboard_set_contents (GtkClipboard         *clipboard,
  * @n_targets:  number of elements in @targets
  * @get_func:   function to call to get the actual clipboard data
  * @clear_func: when the clipboard contents are set again, this function will
- *              be called, and get_func will not be subsequently called.
+ *              be called, and @get_func will not be subsequently called.
  * @user_data:  user data to pass to @get_func and @clear_func.
  * 
- * Virtually set the contents of the specified clipboard by providing
+ * Virtually sets the contents of the specified clipboard by providing
  * a list of supported formats for the clipboard data and a function
  * to call to get the actual data when it is requested.
  * 
@@ -395,17 +395,17 @@ gtk_clipboard_set_with_data (GtkClipboard          *clipboard,
  * @n_targets:  number of elements in @targets
  * @get_func:   function to call to get the actual clipboard data
  * @clear_func: when the clipboard contents are set again, this function will
- *              be called, and get_func will not be subsequently called.
+ *              be called, and @get_func will not be subsequently called.
  * @owner:      an object that "owns" the data. This object will be passed
  *              to the callbacks when called. 
  * 
- * Virtually set the contents of the specified clipboard by providing
+ * Virtually sets the contents of the specified clipboard by providing
  * a list of supported formats for the clipboard data and a function
  * to call to get the actual data when it is requested.
  *
  * The difference between this function and gtk_clipboard_set_with_data
  * is that instead of an generic @user_data pointer, a #GObject is passed
- * in. Because of this, 
+ * in. 
  * 
  * Return value: %TRUE if setting the clipboard data succeeded. If setting
  *               the clipboard data failed the provided callback functions
@@ -477,7 +477,7 @@ clipboard_unset (GtkClipboard *clipboard)
  * gtk_clipboard_clear:
  * @clipboard:  a #GtkClipboard
  * 
- * Clear the contents of the clipboard. Generally this should only
+ * Clears the contents of the clipboard. Generally this should only
  * be called between the time you call gtk_clipboard_set_contents(),
  * and when the @clear_func you supplied is called. Otherwise, the
  * clipboard may be owned by someone else.
@@ -515,7 +515,7 @@ text_clear_func (GtkClipboard *clipboard,
  * @len:       length of @text, in bytes, or -1, in which case
  *             the length will be determined with strlen().
  * 
- * Set the contents of the clipboard to the given UTF-8 string. GTK+ will
+ * Sets the contents of the clipboard to the given UTF-8 string. GTK+ will
  * make a copy of the text and take responsibility for responding
  * for requests for the text, and for converting the text into
  * the requested format.
@@ -590,7 +590,7 @@ selection_received (GtkWidget            *widget,
  * @target:    an atom representing the form into which the clipboard
  *             owner should convert the selection.
  * @callback:  A function to call when the results are received
- *             (or the retrieval fails.) If the retrieval fails
+ *             (or the retrieval fails). If the retrieval fails
  *             the length field of @selection_data will be
  *             negative.
  * @user_data: user data to pass to @callback
@@ -786,7 +786,7 @@ clipboard_text_received_func (GtkClipboard *clipboard,
  * the data to be received using the main loop, so events,
  * timeouts, etc, may be dispatched during the wait.
  * 
- * Return value: a newly allocated UTF-8 string which must
+ * Return value: a newly-allocated UTF-8 string which must
  *               be freed with g_free(), or %NULL if retrieving
  *               the selection data failed. (This could happen
  *               for various reasons, in particular if the
@@ -820,3 +820,35 @@ gtk_clipboard_wait_for_text (GtkClipboard *clipboard)
   return results.data;
 }
 
+/**
+ * gtk_clipboard_wait_is_text_available:
+ * @clipboard: a #GtkClipboard
+ * 
+ * Test to see if there is text available to be pasted
+ * This is done by requesting the TARGETS atom and checking
+ * if it contains any of the names: STRING, TEXT, COMPOUND_TEXT,
+ * UTF8_STRING. This function waits for the data to be received
+ * using the main loop, so events, timeouts, etc, may be dispatched
+ * during the wait.
+ *
+ * This function is a little faster than calling
+ * gtk_clipboard_wait_for_text() since it doesn't need to retrieve
+ * the actual text.
+ * 
+ * Return value: %TRUE is there is text available, %FALSE otherwise.
+ **/
+gboolean
+gtk_clipboard_wait_is_text_available (GtkClipboard *clipboard)
+{
+  GtkSelectionData *data;
+  gboolean result = FALSE;
+
+  data = gtk_clipboard_wait_for_contents (clipboard, gdk_atom_intern ("TARGETS", FALSE));
+  if (data)
+    {
+      result = gtk_selection_data_targets_include_text (data);
+      gtk_selection_data_free (data);
+    }
+
+  return result;
+}
