@@ -25,22 +25,21 @@
  */
 
 #include "gtkfixed.h"
+#include "gtkintl.h"
 
+enum {
+  CHILD_PROP_0,
+  CHILD_PROP_X,
+  CHILD_PROP_Y
+};
 
 static void gtk_fixed_class_init    (GtkFixedClass    *klass);
 static void gtk_fixed_init          (GtkFixed         *fixed);
-static void gtk_fixed_map           (GtkWidget        *widget);
 static void gtk_fixed_realize       (GtkWidget        *widget);
 static void gtk_fixed_size_request  (GtkWidget        *widget,
 				     GtkRequisition   *requisition);
 static void gtk_fixed_size_allocate (GtkWidget        *widget,
 				     GtkAllocation    *allocation);
-static void gtk_fixed_paint         (GtkWidget        *widget,
-				     GdkRectangle     *area);
-static void gtk_fixed_draw          (GtkWidget        *widget,
-				     GdkRectangle     *area);
-static gint gtk_fixed_expose        (GtkWidget        *widget,
-				     GdkEventExpose   *event);
 static void gtk_fixed_add           (GtkContainer     *container,
 				     GtkWidget        *widget);
 static void gtk_fixed_remove        (GtkContainer     *container,
@@ -51,6 +50,16 @@ static void gtk_fixed_forall        (GtkContainer     *container,
 				     gpointer          callback_data);
 static GtkType gtk_fixed_child_type (GtkContainer     *container);
 
+static void gtk_fixed_set_child_property (GtkContainer *container,
+                                          GtkWidget    *child,
+                                          guint         property_id,
+                                          const GValue *value,
+                                          GParamSpec   *pspec);
+static void gtk_fixed_get_child_property (GtkContainer *container,
+                                          GtkWidget    *child,
+                                          guint         property_id,
+                                          GValue       *value,
+                                          GParamSpec   *pspec);
 
 static GtkContainerClass *parent_class = NULL;
 
@@ -93,17 +102,37 @@ gtk_fixed_class_init (GtkFixedClass *class)
 
   parent_class = gtk_type_class (GTK_TYPE_CONTAINER);
 
-  widget_class->map = gtk_fixed_map;
   widget_class->realize = gtk_fixed_realize;
   widget_class->size_request = gtk_fixed_size_request;
   widget_class->size_allocate = gtk_fixed_size_allocate;
-  widget_class->draw = gtk_fixed_draw;
-  widget_class->expose_event = gtk_fixed_expose;
 
   container_class->add = gtk_fixed_add;
   container_class->remove = gtk_fixed_remove;
   container_class->forall = gtk_fixed_forall;
   container_class->child_type = gtk_fixed_child_type;
+
+  container_class->set_child_property = gtk_fixed_set_child_property;
+  container_class->get_child_property = gtk_fixed_get_child_property;
+
+  gtk_container_class_install_child_property (container_class,
+					      CHILD_PROP_X,
+					      g_param_spec_int ("x",
+                                                                _("X position"),
+                                                                _("X position of child widget"),
+                                                                G_MININT,
+                                                                G_MAXINT,
+                                                                0,
+                                                                G_PARAM_READWRITE));
+
+  gtk_container_class_install_child_property (container_class,
+					      CHILD_PROP_Y,
+					      g_param_spec_int ("y",
+                                                                _("Y position"),
+                                                                _("Y position of child widget"),
+                                                                G_MININT,
+                                                                G_MAXINT,
+                                                                0,
+                                                                G_PARAM_READWRITE));
 }
 
 static GtkType
@@ -129,15 +158,35 @@ gtk_fixed_new (void)
   return GTK_WIDGET (fixed);
 }
 
+static GtkFixedChild*
+get_child (GtkFixed  *fixed,
+           GtkWidget *widget)
+{
+  GList *children;
+  
+  children = fixed->children;
+  while (children)
+    {
+      GtkFixedChild *child;
+      
+      child = children->data;
+      children = children->next;
+
+      if (child->widget == widget)
+        return child;
+    }
+
+  return NULL;
+}
+
 void
 gtk_fixed_put (GtkFixed       *fixed,
                GtkWidget      *widget,
-               gint16         x,
-               gint16         y)
+               gint            x,
+               gint            y)
 {
   GtkFixedChild *child_info;
 
-  g_return_if_fail (fixed != NULL);
   g_return_if_fail (GTK_IS_FIXED (fixed));
   g_return_if_fail (widget != NULL);
 
@@ -148,49 +197,109 @@ gtk_fixed_put (GtkFixed       *fixed,
 
   gtk_widget_set_parent (widget, GTK_WIDGET (fixed));
 
-  fixed->children = g_list_append (fixed->children, child_info); 
+  fixed->children = g_list_append (fixed->children, child_info);
+}
 
-  if (GTK_WIDGET_REALIZED (fixed))
-    gtk_widget_realize (widget);
+static void
+gtk_fixed_move_internal (GtkFixed       *fixed,
+                         GtkWidget      *widget,
+                         gboolean        change_x,
+                         gint            x,
+                         gboolean        change_y,
+                         gint            y)
+{
+  GtkFixedChild *child;
+  
+  g_return_if_fail (GTK_IS_FIXED (fixed));
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+  g_return_if_fail (widget->parent == GTK_WIDGET (fixed));  
+  
+  child = get_child (fixed, widget);
 
-  if (GTK_WIDGET_VISIBLE (fixed) && GTK_WIDGET_VISIBLE (widget))
+  g_assert (child);
+
+  gtk_widget_freeze_child_notify (widget);
+  
+  if (change_x)
     {
-      if (GTK_WIDGET_MAPPED (fixed))
-	gtk_widget_map (widget);
-      
-      gtk_widget_queue_resize (GTK_WIDGET (fixed));
+      child->x = x;
+      gtk_widget_child_notify (widget, "x");
     }
+
+  if (change_y)
+    {
+      child->y = y;
+      gtk_widget_child_notify (widget, "y");
+    }
+
+  gtk_widget_thaw_child_notify (widget);
+  
+  if (GTK_WIDGET_VISIBLE (widget) && GTK_WIDGET_VISIBLE (fixed))
+    gtk_widget_queue_resize (GTK_WIDGET (fixed));
 }
 
 void
 gtk_fixed_move (GtkFixed       *fixed,
                 GtkWidget      *widget,
-                gint16         x,
-                gint16         y)
+                gint            x,
+                gint            y)
 {
-  GtkFixedChild *child;
-  GList *children;
-
-  g_return_if_fail (fixed != NULL);
   g_return_if_fail (GTK_IS_FIXED (fixed));
-  g_return_if_fail (widget != NULL);
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+  g_return_if_fail (widget->parent == GTK_WIDGET (fixed));
 
-  children = fixed->children;
-  while (children)
+  gtk_fixed_move_internal (fixed, widget, TRUE, x, TRUE, y);
+}
+
+static void
+gtk_fixed_set_child_property (GtkContainer    *container,
+                              GtkWidget       *child,
+                              guint            property_id,
+                              const GValue    *value,
+                              GParamSpec      *pspec)
+{
+  switch (property_id)
     {
-      child = children->data;
-      children = children->next;
+    case CHILD_PROP_X:
+      gtk_fixed_move_internal (GTK_FIXED (container),
+                               child,
+                               TRUE, g_value_get_int (value),
+                               FALSE, 0);
+      break;
+    case CHILD_PROP_Y:
+      gtk_fixed_move_internal (GTK_FIXED (container),
+                               child,
+                               FALSE, 0,
+                               TRUE, g_value_get_int (value));
+      break;
+    default:
+      GTK_CONTAINER_WARN_INVALID_CHILD_PROPERTY_ID (container, property_id, pspec);
+      break;
+    }
+}
 
-      if (child->widget == widget)
-        {
-          child->x = x;
-          child->y = y;
+static void
+gtk_fixed_get_child_property (GtkContainer *container,
+                              GtkWidget    *child,
+                              guint         property_id,
+                              GValue       *value,
+                              GParamSpec   *pspec)
+{
+  GtkFixedChild *fixed_child;
 
-          if (GTK_WIDGET_VISIBLE (widget) && GTK_WIDGET_VISIBLE (fixed))
-            gtk_widget_queue_resize (GTK_WIDGET (fixed));
-
-          break;
-        }
+  fixed_child = get_child (GTK_FIXED (container), child);
+  
+  switch (property_id)
+    {
+    case CHILD_PROP_X:
+      g_value_set_int (value, fixed_child->x);
+      break;
+    case CHILD_PROP_Y:
+      g_value_set_int (value, fixed_child->y);
+      break;
+    default:
+      GTK_CONTAINER_WARN_INVALID_CHILD_PROPERTY_ID (container, property_id, pspec);
+      break;
     }
 }
 
@@ -201,9 +310,6 @@ gtk_fixed_map (GtkWidget *widget)
   GtkFixedChild *child;
   GList *children;
 
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_FIXED (widget));
-
   GTK_WIDGET_SET_FLAGS (widget, GTK_MAPPED);
   fixed = GTK_FIXED (widget);
 
@@ -227,7 +333,6 @@ gtk_fixed_realize (GtkWidget *widget)
   GdkWindowAttr attributes;
   gint attributes_mask;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_FIXED (widget));
 
   GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
@@ -262,7 +367,6 @@ gtk_fixed_size_request (GtkWidget      *widget,
   GList *children;
   GtkRequisition child_requisition;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_FIXED (widget));
   g_return_if_fail (requisition != NULL);
 
@@ -304,7 +408,6 @@ gtk_fixed_size_allocate (GtkWidget     *widget,
   GList *children;
   guint16 border_width;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_FIXED(widget));
   g_return_if_fail (allocation != NULL);
 
@@ -338,89 +441,10 @@ gtk_fixed_size_allocate (GtkWidget     *widget,
     }
 }
 
-static void
-gtk_fixed_paint (GtkWidget    *widget,
-		 GdkRectangle *area)
-{
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_FIXED (widget));
-  g_return_if_fail (area != NULL);
-
-  if (GTK_WIDGET_DRAWABLE (widget))
-    gdk_window_clear_area (widget->window,
-			   area->x, area->y,
-			   area->width, area->height);
-}
-
-static void
-gtk_fixed_draw (GtkWidget    *widget,
-		GdkRectangle *area)
-{
-  GtkFixed *fixed;
-  GtkFixedChild *child;
-  GdkRectangle child_area;
-  GList *children;
-
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_FIXED (widget));
-
-  if (GTK_WIDGET_DRAWABLE (widget))
-    {
-      fixed = GTK_FIXED (widget);
-      gtk_fixed_paint (widget, area);
-
-      children = fixed->children;
-      while (children)
-	{
-	  child = children->data;
-	  children = children->next;
-
-	  if (gtk_widget_intersect (child->widget, area, &child_area))
-	    gtk_widget_draw (child->widget, &child_area);
-	}
-    }
-}
-
-static gint
-gtk_fixed_expose (GtkWidget      *widget,
-		  GdkEventExpose *event)
-{
-  GtkFixed *fixed;
-  GtkFixedChild *child;
-  GdkEventExpose child_event;
-  GList *children;
-
-  g_return_val_if_fail (widget != NULL, FALSE);
-  g_return_val_if_fail (GTK_IS_FIXED (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-
-  if (GTK_WIDGET_DRAWABLE (widget))
-    {
-      fixed = GTK_FIXED (widget);
-
-      child_event = *event;
-
-      children = fixed->children;
-      while (children)
-	{
-	  child = children->data;
-	  children = children->next;
-
-	  if (GTK_WIDGET_NO_WINDOW (child->widget) &&
-	      gtk_widget_intersect (child->widget, &event->area, 
-				    &child_event.area))
-	    gtk_widget_event (child->widget, (GdkEvent*) &child_event);
-	}
-    }
-
-  return FALSE;
-}
-
 static void
 gtk_fixed_add (GtkContainer *container,
 	       GtkWidget    *widget)
 {
-  g_return_if_fail (container != NULL);
   g_return_if_fail (GTK_IS_FIXED (container));
   g_return_if_fail (widget != NULL);
 
@@ -435,7 +459,6 @@ gtk_fixed_remove (GtkContainer *container,
   GtkFixedChild *child;
   GList *children;
 
-  g_return_if_fail (container != NULL);
   g_return_if_fail (GTK_IS_FIXED (container));
   g_return_if_fail (widget != NULL);
 
@@ -476,7 +499,6 @@ gtk_fixed_forall (GtkContainer *container,
   GtkFixedChild *child;
   GList *children;
 
-  g_return_if_fail (container != NULL);
   g_return_if_fail (GTK_IS_FIXED (container));
   g_return_if_fail (callback != NULL);
 
