@@ -38,6 +38,7 @@ enum {
   PROP_LABEL_XALIGN,
   PROP_LABEL_YALIGN,
   PROP_SHADOW,
+  PROP_SHADOW_TYPE,
   PROP_LABEL_WIDGET
 };
 
@@ -60,8 +61,6 @@ static void gtk_frame_size_request  (GtkWidget      *widget,
 				     GtkRequisition *requisition);
 static void gtk_frame_size_allocate (GtkWidget      *widget,
 				     GtkAllocation  *allocation);
-static void gtk_frame_map           (GtkWidget      *widget);
-static void gtk_frame_unmap         (GtkWidget      *widget);
 static void gtk_frame_remove        (GtkContainer   *container,
 				     GtkWidget      *child);
 static void gtk_frame_forall        (GtkContainer   *container,
@@ -150,7 +149,14 @@ gtk_frame_class_init (GtkFrameClass *class)
                                                         G_PARAM_WRITABLE));
   g_object_class_install_property (gobject_class,
                                    PROP_SHADOW,
-                                   g_param_spec_enum ("shadow",
+                                   g_param_spec_enum ("shadow", NULL,
+                                                      _("Deprecated property, use shadow_type instead."),
+						      GTK_TYPE_SHADOW_TYPE,
+						      GTK_SHADOW_ETCHED_IN,
+                                                      G_PARAM_READABLE | G_PARAM_WRITABLE));
+  g_object_class_install_property (gobject_class,
+                                   PROP_SHADOW_TYPE,
+                                   g_param_spec_enum ("shadow_type",
                                                       _("Frame shadow"),
                                                       _("Appearance of the frame border."),
 						      GTK_TYPE_SHADOW_TYPE,
@@ -168,8 +174,6 @@ gtk_frame_class_init (GtkFrameClass *class)
   widget_class->expose_event = gtk_frame_expose;
   widget_class->size_request = gtk_frame_size_request;
   widget_class->size_allocate = gtk_frame_size_allocate;
-  widget_class->map = gtk_frame_map;
-  widget_class->unmap = gtk_frame_unmap;
 
   container_class->remove = gtk_frame_remove;
   container_class->forall = gtk_frame_forall;
@@ -210,6 +214,7 @@ gtk_frame_set_property (GObject         *object,
 				 g_value_get_double (value));
       break;
     case PROP_SHADOW:
+    case PROP_SHADOW_TYPE:
       gtk_frame_set_shadow_type (frame, g_value_get_enum (value));
       break;
     case PROP_LABEL_WIDGET:
@@ -243,6 +248,7 @@ gtk_frame_get_property (GObject         *object,
       g_value_set_double (value, frame->label_yalign);
       break;
     case PROP_SHADOW:
+    case PROP_SHADOW_TYPE:
       g_value_set_enum (value, frame->shadow_type);
       break;
     case PROP_LABEL_WIDGET:
@@ -300,7 +306,6 @@ void
 gtk_frame_set_label (GtkFrame *frame,
 		     const gchar *label)
 {
-  g_return_if_fail (frame != NULL);
   g_return_if_fail (GTK_IS_FRAME (frame));
 
   if (!label)
@@ -334,7 +339,6 @@ gtk_frame_set_label (GtkFrame *frame,
 G_CONST_RETURN gchar *
 gtk_frame_get_label (GtkFrame *frame)
 {
-  g_return_val_if_fail (frame != NULL, NULL);
   g_return_val_if_fail (GTK_IS_FRAME (frame), NULL);
 
   if (frame->label_widget && GTK_IS_LABEL (frame->label_widget))
@@ -358,7 +362,6 @@ gtk_frame_set_label_widget (GtkFrame  *frame,
 {
   gboolean need_resize = FALSE;
   
-  g_return_if_fail (frame != NULL);
   g_return_if_fail (GTK_IS_FRAME (frame));
   g_return_if_fail (label_widget == NULL || GTK_IS_WIDGET (label_widget));
   g_return_if_fail (label_widget == NULL || label_widget->parent == NULL);
@@ -387,12 +390,28 @@ gtk_frame_set_label_widget (GtkFrame  *frame,
   g_object_notify (G_OBJECT (frame), "label_widget");
 }
 
+/**
+ * gtk_frame_get_label_widget:
+ * @frame: a #GtkFrame
+ *
+ * Retrieves the label widget for the frame. See
+ * gtk_frame_set_label_widget().
+ *
+ * Return value: the label widget, or %NULL if there is none.
+ **/
+GtkWidget *
+gtk_frame_get_label_widget (GtkFrame *frame)
+{
+  g_return_val_if_fail (GTK_IS_FRAME (frame), NULL);
+
+  return frame->label_widget;
+}
+
 void
 gtk_frame_set_label_align (GtkFrame *frame,
 			   gfloat    xalign,
 			   gfloat    yalign)
 {
-  g_return_if_fail (frame != NULL);
   g_return_if_fail (GTK_IS_FRAME (frame));
 
   xalign = CLAMP (xalign, 0.0, 1.0);
@@ -413,17 +432,38 @@ gtk_frame_set_label_align (GtkFrame *frame,
   gtk_widget_queue_resize (GTK_WIDGET (frame));
 }
 
+/**
+ * gtk_frame_get_label_align:
+ * @frame: a #GtkFrame
+ * @xalign: location to store X alignment of frame's label, or %NULL
+ * @yalign: location to store X alignment of frame's label, or %NULL
+ * 
+ * Retrieves the X and Y alignment of the frame's label. See
+ * gtk_frame_set_label_align().
+ **/
+void
+gtk_frame_get_label_align (GtkFrame *frame,
+		           gfloat   *xalign,
+			   gfloat   *yalign)
+{
+  g_return_if_fail (GTK_IS_FRAME (frame));
+
+  if (xalign)
+    *xalign = frame->label_xalign;
+  if (yalign)
+    *yalign = frame->label_yalign;
+}
+
 void
 gtk_frame_set_shadow_type (GtkFrame      *frame,
 			   GtkShadowType  type)
 {
-  g_return_if_fail (frame != NULL);
   g_return_if_fail (GTK_IS_FRAME (frame));
 
   if ((GtkShadowType) frame->shadow_type != type)
     {
       frame->shadow_type = type;
-      g_object_notify (G_OBJECT (frame), "shadow");
+      g_object_notify (G_OBJECT (frame), "shadow_type");
 
       if (GTK_WIDGET_DRAWABLE (frame))
 	{
@@ -434,6 +474,23 @@ gtk_frame_set_shadow_type (GtkFrame      *frame,
     }
 }
 
+/**
+ * gtk_frame_get_shadow_type:
+ * @frame: a #GtkFrame
+ *
+ * Retrieves the shadow type of the frame. See
+ * gtk_frame_set_shadow_type().
+ *
+ * Return value: the current shadow type of the frame.
+ **/
+GtkShadowType
+gtk_frame_get_shadow_type (GtkFrame *frame)
+{
+  g_return_val_if_fail (GTK_IS_FRAME (frame), GTK_SHADOW_ETCHED_IN);
+
+  return frame->shadow_type;
+}
+
 static void
 gtk_frame_paint (GtkWidget    *widget,
 		 GdkRectangle *area)
@@ -587,37 +644,10 @@ gtk_frame_size_allocate (GtkWidget     *widget,
     }
 }
 
-static void
-gtk_frame_map (GtkWidget *widget)
-{
-  GtkFrame *frame = GTK_FRAME (widget);
-  
-  if (frame->label_widget &&
-      GTK_WIDGET_VISIBLE (frame->label_widget) &&
-      !GTK_WIDGET_MAPPED (frame->label_widget))
-    gtk_widget_map (frame->label_widget);
-
-  if (GTK_WIDGET_CLASS (parent_class)->map)
-    (* GTK_WIDGET_CLASS (parent_class)->map) (widget);
-}
-
-static void
-gtk_frame_unmap (GtkWidget *widget)
-{
-  GtkFrame *frame = GTK_FRAME (widget);
-
-  if (GTK_WIDGET_CLASS (parent_class)->unmap)
-    (* GTK_WIDGET_CLASS (parent_class)->unmap) (widget);
-
-  if (frame->label_widget && GTK_WIDGET_MAPPED (frame->label_widget))
-    gtk_widget_unmap (frame->label_widget);
-}
-
 static void
 gtk_frame_compute_child_allocation (GtkFrame      *frame,
 				    GtkAllocation *child_allocation)
 {
-  g_return_if_fail (frame != NULL);
   g_return_if_fail (GTK_IS_FRAME (frame));
   g_return_if_fail (child_allocation != NULL);
 
