@@ -47,19 +47,22 @@ gtk_im_context_get_type (void)
 
   if (!im_context_type)
     {
-      static const GtkTypeInfo im_context_info =
+      static const GTypeInfo im_context_info =
       {
-	"GtkIMContext",
-	sizeof (GtkIMContext),
-	sizeof (GtkIMContextClass),
-	(GtkClassInitFunc) gtk_im_context_class_init,
-	(GtkObjectInitFunc) gtk_im_context_init,
-	/* reserved_1 */ NULL,
-        /* reserved_2 */ NULL,
-        (GtkClassInitFunc) NULL,
+        sizeof (GtkIMContextClass),
+        (GBaseInitFunc) NULL,
+        (GBaseFinalizeFunc) NULL,
+        (GClassInitFunc) gtk_im_context_class_init,
+        NULL,           /* class_finalize */
+        NULL,           /* class_data */
+        sizeof (GtkIMContext),
+        0,              /* n_preallocs */
+        (GInstanceInitFunc) gtk_im_context_init,
       };
-
-      im_context_type = gtk_type_unique (GTK_TYPE_OBJECT, &im_context_info);
+      
+      im_context_type = g_type_register_static (G_TYPE_OBJECT,
+						"GtkIMContext",
+						&im_context_info, 0);
     }
 
   return im_context_type;
@@ -152,7 +155,6 @@ gtk_im_context_set_client_window (GtkIMContext *context,
 {
   GtkIMContextClass *klass;
   
-  g_return_if_fail (context != NULL);
   g_return_if_fail (GTK_IS_IM_CONTEXT (context));
 
   klass = GTK_IM_CONTEXT_GET_CLASS (context);
@@ -184,7 +186,6 @@ gtk_im_context_get_preedit_string (GtkIMContext   *context,
 {
   GtkIMContextClass *klass;
   
-  g_return_if_fail (context != NULL);
   g_return_if_fail (GTK_IS_IM_CONTEXT (context));
   
   klass = GTK_IM_CONTEXT_GET_CLASS (context);
@@ -210,7 +211,6 @@ gtk_im_context_filter_keypress (GtkIMContext *context,
 {
   GtkIMContextClass *klass;
   
-  g_return_val_if_fail (context != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_IM_CONTEXT (context), FALSE);
   g_return_val_if_fail (key != NULL, FALSE);
 
@@ -232,7 +232,6 @@ gtk_im_context_focus_in (GtkIMContext   *context)
 {
   GtkIMContextClass *klass;
   
-  g_return_if_fail (context != NULL);
   g_return_if_fail (GTK_IS_IM_CONTEXT (context));
   
   klass = GTK_IM_CONTEXT_GET_CLASS (context);
@@ -254,7 +253,6 @@ gtk_im_context_focus_out (GtkIMContext   *context)
 {
   GtkIMContextClass *klass;
   
-  g_return_if_fail (context != NULL);
   g_return_if_fail (GTK_IS_IM_CONTEXT (context));
 
   klass = GTK_IM_CONTEXT_GET_CLASS (context);
@@ -275,7 +273,6 @@ gtk_im_context_reset (GtkIMContext   *context)
 {
   GtkIMContextClass *klass;
   
-  g_return_if_fail (context != NULL);
   g_return_if_fail (GTK_IS_IM_CONTEXT (context));
 
   klass = GTK_IM_CONTEXT_GET_CLASS (context);
@@ -298,7 +295,6 @@ gtk_im_context_set_cursor_location (GtkIMContext *context,
 {
   GtkIMContextClass *klass;
   
-  g_return_if_fail (context != NULL);
   g_return_if_fail (GTK_IS_IM_CONTEXT (context));
 
   klass = GTK_IM_CONTEXT_GET_CLASS (context);
@@ -322,7 +318,6 @@ gtk_im_context_set_use_preedit (GtkIMContext *context,
 {
   GtkIMContextClass *klass;
   
-  g_return_if_fail (context != NULL);
   g_return_if_fail (GTK_IS_IM_CONTEXT (context));
 
   klass = GTK_IM_CONTEXT_GET_CLASS (context);
