@@ -227,7 +227,7 @@ gtk_im_module_init ()
 {
   GString *line_buf = g_string_new (NULL);
   GString *tmp_buf = g_string_new (NULL);
-  const gchar *filename = gtk_rc_get_im_module_file();
+  gchar *filename = gtk_rc_get_im_module_file();
   FILE *file;
   gboolean have_error = FALSE;
 
@@ -441,20 +441,24 @@ _gtk_im_module_create (const gchar *context_id)
 
 /* Match @locale against @against.
  * 
- * 'en_US' against 'en_US'       => 3
- * 'en_US' against 'en'          => 2
- * 'en', 'en_UK' against 'en_US' => 1
+ * 'en_US' against 'en_US'       => 4
+ * 'en_US' against 'en'          => 3
+ * 'en', 'en_UK' against 'en_US' => 2
+ *  all locales, against '*' 	 => 1
  */
 static gint
 match_locale (const gchar *locale,
 	      const gchar *against,
 	      gint         against_len)
 {
+  if (strcmp (against, "*") == 0)
+    return 1;
+
   if (strcmp (locale, against) == 0)
-    return 3;
+    return 4;
 
   if (strncmp (locale, against, 2) == 0)
-    return (against_len == 2) ? 2 : 1;
+    return (against_len == 2) ? 3 : 2;
 
   return 0;
 }
@@ -478,7 +482,7 @@ _gtk_im_module_get_default_context_id (const gchar *locale)
   gint best_goodness = 0;
   gint i;
   gchar *tmp_locale, *tmp;
-  gchar *envvar;
+  const gchar *envvar;
       
   if (!contexts_hash)
     gtk_im_module_init ();
