@@ -51,49 +51,72 @@ enum {
    PROP_HEIGHT
 };
 
-static void     gtk_layout_class_init         (GtkLayoutClass *class);
-static void     gtk_layout_get_property       (GObject        *object,
-					       guint          prop_id,
-					       GValue         *value,
-					       GParamSpec     *pspec);
-static void     gtk_layout_set_property       (GObject        *object,
-					       guint          prop_id,
-					       const GValue   *value,
-					       GParamSpec     *pspec);
-static void     gtk_layout_init               (GtkLayout      *layout);
+enum {
+  CHILD_PROP_0,
+  CHILD_PROP_X,
+  CHILD_PROP_Y
+};
 
-static void     gtk_layout_finalize           (GObject        *object);
-static void     gtk_layout_realize            (GtkWidget      *widget);
-static void     gtk_layout_unrealize          (GtkWidget      *widget);
-static void     gtk_layout_map                (GtkWidget      *widget);
-static void     gtk_layout_size_request       (GtkWidget      *widget,
-					       GtkRequisition *requisition);
-static void     gtk_layout_size_allocate      (GtkWidget      *widget,
-					       GtkAllocation  *allocation);
-static gint     gtk_layout_expose             (GtkWidget      *widget, 
-					       GdkEventExpose *event);
+static void gtk_layout_class_init         (GtkLayoutClass *class);
+static void gtk_layout_get_property       (GObject        *object,
+                                           guint           prop_id,
+                                           GValue         *value,
+                                           GParamSpec     *pspec);
+static void gtk_layout_set_property       (GObject        *object,
+                                           guint           prop_id,
+                                           const GValue   *value,
+                                           GParamSpec     *pspec);
+static void gtk_layout_init               (GtkLayout      *layout);
+static void gtk_layout_finalize           (GObject        *object);
+static void gtk_layout_realize            (GtkWidget      *widget);
+static void gtk_layout_unrealize          (GtkWidget      *widget);
+static void gtk_layout_map                (GtkWidget      *widget);
+static void gtk_layout_size_request       (GtkWidget      *widget,
+                                           GtkRequisition *requisition);
+static void gtk_layout_size_allocate      (GtkWidget      *widget,
+                                           GtkAllocation  *allocation);
+static gint gtk_layout_expose             (GtkWidget      *widget,
+                                           GdkEventExpose *event);
+static void gtk_layout_remove             (GtkContainer   *container,
+                                           GtkWidget      *widget);
+static void gtk_layout_forall             (GtkContainer   *container,
+                                           gboolean        include_internals,
+                                           GtkCallback     callback,
+                                           gpointer        callback_data);
+static void gtk_layout_set_adjustments    (GtkLayout      *layout,
+                                           GtkAdjustment  *hadj,
+                                           GtkAdjustment  *vadj);
+static void gtk_layout_set_child_property (GtkContainer   *container,
+                                           GtkWidget      *child,
+                                           guint           property_id,
+                                           const GValue   *value,
+                                           GParamSpec     *pspec);
+static void gtk_layout_get_child_property (GtkContainer   *container,
+                                           GtkWidget      *child,
+                                           guint           property_id,
+                                           GValue         *value,
+                                           GParamSpec     *pspec);
+static void gtk_layout_allocate_child     (GtkLayout      *layout,
+                                           GtkLayoutChild *child);
+static void gtk_layout_adjustment_changed (GtkAdjustment  *adjustment,
+                                           GtkLayout      *layout);
 
-static void     gtk_layout_remove             (GtkContainer *container, 
-					       GtkWidget    *widget);
-static void     gtk_layout_forall             (GtkContainer *container,
-					       gboolean      include_internals,
-					       GtkCallback   callback,
-					       gpointer      callback_data);
-static void     gtk_layout_set_adjustments    (GtkLayout     *layout,
-					       GtkAdjustment *hadj,
-					       GtkAdjustment *vadj);
-
-static void     gtk_layout_allocate_child     (GtkLayout      *layout,
-					       GtkLayoutChild *child);
-
-
-static void     gtk_layout_adjustment_changed (GtkAdjustment  *adjustment,
-					       GtkLayout      *layout);
 
 static GtkWidgetClass *parent_class = NULL;
 
 /* Public interface
  */
+/**
+ * gtk_layout_new:
+ * @hadjustment: horizontal scroll adjustment, or %NULL
+ * @vadjustment: vertical scroll adjustment, or %NULL
+ * 
+ * Creates a new #GtkLayout. Unless you have a specific adjustment
+ * you'd like the layout to use for scrolling, pass %NULL for
+ * @hadjustment and @vadjustment.
+ * 
+ * Return value: a new #GtkLayout
+ **/
   
 GtkWidget*    
 gtk_layout_new (GtkAdjustment *hadjustment,
@@ -108,18 +131,42 @@ gtk_layout_new (GtkAdjustment *hadjustment,
   return GTK_WIDGET (layout);
 }
 
+/**
+ * gtk_layout_get_hadjustment:
+ * @layout: a #GtkLayout
+ * 
+ * This function should only be called after the layout has been
+ * placed in a #GtkScrolledWindow or otherwise configured for
+ * scrolling. It returns the #GtkAdjustment used for communication
+ * between the horizontal scrollbar and @layout.
+ *
+ * See #GtkScrolledWindow, #GtkScrollbar, #GtkAdjustment for details.
+ * 
+ * Return value: horizontal scroll adjustment
+ **/
 GtkAdjustment* 
 gtk_layout_get_hadjustment (GtkLayout     *layout)
 {
-  g_return_val_if_fail (layout != NULL, NULL);
   g_return_val_if_fail (GTK_IS_LAYOUT (layout), NULL);
 
   return layout->hadjustment;
 }
+/**
+ * gtk_layout_get_vadjustment:
+ * @layout: a #GtkLayout
+ * 
+ * This function should only be called after the layout has been
+ * placed in a #GtkScrolledWindow or otherwise configured for
+ * scrolling. It returns the #GtkAdjustment used for communication
+ * between the vertical scrollbar and @layout.
+ *
+ * See #GtkScrolledWindow, #GtkScrollbar, #GtkAdjustment for details.
+ * 
+ * Return value: vertical scroll adjustment
+ **/
 GtkAdjustment* 
 gtk_layout_get_vadjustment (GtkLayout     *layout)
 {
-  g_return_val_if_fail (layout != NULL, NULL);
   g_return_val_if_fail (GTK_IS_LAYOUT (layout), NULL);
 
   return layout->vadjustment;
@@ -132,7 +179,6 @@ gtk_layout_set_adjustments (GtkLayout     *layout,
 {
   gboolean need_adjust = FALSE;
 
-  g_return_if_fail (layout != NULL);
   g_return_if_fail (GTK_IS_LAYOUT (layout));
 
   if (hadj)
@@ -195,30 +241,78 @@ gtk_layout_finalize (GObject *object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+/**
+ * gtk_layout_set_hadjustment:
+ * @layout: a #GtkLayout
+ * @adjustment: new scroll adjustment
+ *
+ * Sets the horizontal scroll adjustment for the layout.
+ *
+ * See #GtkScrolledWindow, #GtkScrollbar, #GtkAdjustment for details.
+ * 
+ **/
 void           
 gtk_layout_set_hadjustment (GtkLayout     *layout,
 			    GtkAdjustment *adjustment)
 {
-  g_return_if_fail (layout != NULL);
   g_return_if_fail (GTK_IS_LAYOUT (layout));
 
   gtk_layout_set_adjustments (layout, adjustment, layout->vadjustment);
   g_object_notify (G_OBJECT (layout), "hadjustment");
 }
  
-
+/**
+ * gtk_layout_set_vadjustment:
+ * @layout: a #GtkLayout
+ * @adjustment: new scroll adjustment
+ *
+ * Sets the vertical scroll adjustment for the layout.
+ *
+ * See #GtkScrolledWindow, #GtkScrollbar, #GtkAdjustment for details.
+ * 
+ **/
 void           
 gtk_layout_set_vadjustment (GtkLayout     *layout,
 			    GtkAdjustment *adjustment)
 {
-  g_return_if_fail (layout != NULL);
   g_return_if_fail (GTK_IS_LAYOUT (layout));
   
   gtk_layout_set_adjustments (layout, layout->hadjustment, adjustment);
   g_object_notify (G_OBJECT (layout), "vadjustment");
 }
 
+static GtkLayoutChild*
+get_child (GtkLayout  *layout,
+           GtkWidget  *widget)
+{
+  GList *children;
+  
+  children = layout->children;
+  while (children)
+    {
+      GtkLayoutChild *child;
+      
+      child = children->data;
+      children = children->next;
 
+      if (child->widget == widget)
+        return child;
+    }
+
+  return NULL;
+}
+
+/**
+ * gtk_layout_put:
+ * @layout: a #GtkLayout
+ * @child_widget: child widget
+ * @x: X position of child widget
+ * @y: Y position of child widget
+ *
+ * Adds @child_widget to @layout, at position (@x,@y).
+ * @layout becomes the new parent container of @child_widget.
+ * 
+ **/
 void           
 gtk_layout_put (GtkLayout     *layout, 
 		GtkWidget     *child_widget, 
@@ -227,9 +321,7 @@ gtk_layout_put (GtkLayout     *layout,
 {
   GtkLayoutChild *child;
 
-  g_return_if_fail (layout != NULL);
   g_return_if_fail (GTK_IS_LAYOUT (layout));
-  g_return_if_fail (child_widget != NULL);
   g_return_if_fail (GTK_IS_WIDGET (child_widget));
   
   child = g_new (GtkLayoutChild, 1);
@@ -240,51 +332,71 @@ gtk_layout_put (GtkLayout     *layout,
 
   layout->children = g_list_append (layout->children, child);
   
-  gtk_widget_set_parent (child_widget, GTK_WIDGET (layout));
   if (GTK_WIDGET_REALIZED (layout))
     gtk_widget_set_parent_window (child->widget, layout->bin_window);
 
-  if (GTK_WIDGET_REALIZED (layout))
-    gtk_widget_realize (child_widget);
-    
-  if (GTK_WIDGET_VISIBLE (layout) && GTK_WIDGET_VISIBLE (child_widget))
-    {
-      if (GTK_WIDGET_MAPPED (layout))
-	gtk_widget_map (child_widget);
-
-      gtk_widget_queue_resize (child_widget);
-    }
+  gtk_widget_set_parent (child_widget, GTK_WIDGET (layout));
 }
 
+static void
+gtk_layout_move_internal (GtkLayout       *layout,
+                          GtkWidget       *widget,
+                          gboolean         change_x,
+                          gint             x,
+                          gboolean         change_y,
+                          gint             y)
+{
+  GtkLayoutChild *child;
+  
+  g_return_if_fail (GTK_IS_LAYOUT (layout));
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+  g_return_if_fail (widget->parent == GTK_WIDGET (layout));  
+  
+  child = get_child (layout, widget);
+
+  g_assert (child);
+
+  gtk_widget_freeze_child_notify (widget);
+  
+  if (change_x)
+    {
+      child->x = x;
+      gtk_widget_child_notify (widget, "x");
+    }
+
+  if (change_y)
+    {
+      child->y = y;
+      gtk_widget_child_notify (widget, "y");
+    }
+
+  gtk_widget_thaw_child_notify (widget);
+  
+  if (GTK_WIDGET_VISIBLE (widget) && GTK_WIDGET_VISIBLE (layout))
+    gtk_widget_queue_resize (GTK_WIDGET (layout));
+}
+
+/**
+ * gtk_layout_move:
+ * @layout: a #GtkLayout
+ * @child_widget: a current child of @layout
+ * @x: X position to move to
+ * @y: Y position to move to
+ *
+ * Moves a current child of @layout to a new position.
+ * 
+ **/
 void           
 gtk_layout_move (GtkLayout     *layout, 
 		 GtkWidget     *child_widget, 
 		 gint           x, 
 		 gint           y)
 {
-  GList *tmp_list;
-  GtkLayoutChild *child;
-  
-  g_return_if_fail (layout != NULL);
   g_return_if_fail (GTK_IS_LAYOUT (layout));
+  g_return_if_fail (GTK_IS_WIDGET (child_widget));
+  g_return_if_fail (child_widget->parent == GTK_WIDGET (layout));  
 
-  tmp_list = layout->children;
-  while (tmp_list)
-    {
-      child = tmp_list->data;
-      tmp_list = tmp_list->next;
-
-      if (child->widget == child_widget)
-	{
-	  child->x = x;
-	  child->y = y;
-
-	  if (GTK_WIDGET_VISIBLE (child_widget) && GTK_WIDGET_VISIBLE (layout))
-	    gtk_widget_queue_resize (child_widget);
-
-	  return;
-	}
-    }
+  gtk_layout_move_internal (layout, child_widget, TRUE, x, TRUE, y);
 }
 
 static void
@@ -310,6 +422,15 @@ gtk_layout_set_adjustment_upper (GtkAdjustment *adj,
     }
 }
 
+/**
+ * gtk_layout_set_size:
+ * @layout: a #GtkLayout
+ * @width: width of entire scrollable area
+ * @height: height of entire scrollable area
+ *
+ * Sets the size of the scrollable area of the layout.
+ * 
+ **/
 void
 gtk_layout_set_size (GtkLayout     *layout, 
 		     guint          width,
@@ -317,11 +438,11 @@ gtk_layout_set_size (GtkLayout     *layout,
 {
   GtkWidget *widget;
   
-  g_return_if_fail (layout != NULL);
   g_return_if_fail (GTK_IS_LAYOUT (layout));
 
   widget = GTK_WIDGET (layout);
   
+  g_object_freeze_notify (G_OBJECT (layout));
   if (width != layout->width)
      {
 	layout->width = width;
@@ -332,6 +453,7 @@ gtk_layout_set_size (GtkLayout     *layout,
 	layout->height = height;
 	g_object_notify (G_OBJECT (layout), "height");
      }
+  g_object_thaw_notify (G_OBJECT (layout));
 
   gtk_layout_set_adjustment_upper (layout->hadjustment, layout->width);
   gtk_layout_set_adjustment_upper (layout->vadjustment, layout->height);
@@ -344,19 +466,52 @@ gtk_layout_set_size (GtkLayout     *layout,
     }
 }
 
+/**
+ * gtk_layout_get_size:
+ * @layout: a #GtkLayout
+ * @width: location to store the width set on @layout, or %NULL
+ * @height: location to store the height set on @layout, or %NULL
+ *
+ * Gets the size that has been set on the layout, and that determines
+ * the total extents of the layout's scrollbar area. See
+ * gtk_layout_set_size ().
+ **/
+void
+gtk_layout_get_size (GtkLayout *layout,
+		     guint     *width,
+		     guint     *height)
+{
+  g_return_if_fail (GTK_IS_LAYOUT (layout));
+
+  if (width)
+    *width = layout->width;
+  if (height)
+    *height = layout->height;
+}
+
+/**
+ * gtk_layout_freeze:
+ * @layout: a #GtkLayout
+ * 
+ * This is a deprecated function, it doesn't do anything useful.
+ **/
 void
 gtk_layout_freeze (GtkLayout *layout)
 {
-  g_return_if_fail (layout != NULL);
   g_return_if_fail (GTK_IS_LAYOUT (layout));
 
   layout->freeze_count++;
 }
 
+/**
+ * gtk_layout_thaw:
+ * @layout: a #GtkLayout
+ * 
+ * This is a deprecated function, it doesn't do anything useful.
+ **/
 void
 gtk_layout_thaw (GtkLayout *layout)
 {
-  g_return_if_fail (layout != NULL);
   g_return_if_fail (GTK_IS_LAYOUT (layout));
 
   if (layout->freeze_count)
@@ -411,6 +566,29 @@ gtk_layout_class_init (GtkLayoutClass *class)
   gobject_class->get_property = gtk_layout_get_property;
   gobject_class->finalize = gtk_layout_finalize;
 
+  container_class->set_child_property = gtk_layout_set_child_property;
+  container_class->get_child_property = gtk_layout_get_child_property;
+
+  gtk_container_class_install_child_property (container_class,
+					      CHILD_PROP_X,
+					      g_param_spec_int ("x",
+                                                                _("X position"),
+                                                                _("X position of child widget"),
+                                                                G_MININT,
+                                                                G_MAXINT,
+                                                                0,
+                                                                G_PARAM_READWRITE));
+
+  gtk_container_class_install_child_property (container_class,
+					      CHILD_PROP_Y,
+					      g_param_spec_int ("y",
+                                                                _("Y position"),
+                                                                _("Y position of child widget"),
+                                                                G_MININT,
+                                                                G_MAXINT,
+                                                                0,
+                                                                G_PARAM_READWRITE));
+  
   g_object_class_install_property (gobject_class,
 				   PROP_HADJUSTMENT,
 				   g_param_spec_object ("hadjustment",
@@ -526,6 +704,57 @@ gtk_layout_set_property (GObject      *object,
     }
 }
 
+static void
+gtk_layout_set_child_property (GtkContainer    *container,
+                               GtkWidget       *child,
+                               guint            property_id,
+                               const GValue    *value,
+                               GParamSpec      *pspec)
+{
+  switch (property_id)
+    {
+    case CHILD_PROP_X:
+      gtk_layout_move_internal (GTK_LAYOUT (container),
+                                child,
+                                TRUE, g_value_get_int (value),
+                                FALSE, 0);
+      break;
+    case CHILD_PROP_Y:
+      gtk_layout_move_internal (GTK_LAYOUT (container),
+                                child,
+                                FALSE, 0,
+                                TRUE, g_value_get_int (value));
+      break;
+    default:
+      GTK_CONTAINER_WARN_INVALID_CHILD_PROPERTY_ID (container, property_id, pspec);
+      break;
+    }
+}
+
+static void
+gtk_layout_get_child_property (GtkContainer *container,
+                               GtkWidget    *child,
+                               guint         property_id,
+                               GValue       *value,
+                               GParamSpec   *pspec)
+{
+  GtkLayoutChild *layout_child;
+
+  layout_child = get_child (GTK_LAYOUT (container), child);
+  
+  switch (property_id)
+    {
+    case CHILD_PROP_X:
+      g_value_set_int (value, layout_child->x);
+      break;
+    case CHILD_PROP_Y:
+      g_value_set_int (value, layout_child->y);
+      break;
+    default:
+      GTK_CONTAINER_WARN_INVALID_CHILD_PROPERTY_ID (container, property_id, pspec);
+      break;
+    }
+}
 
 static void
 gtk_layout_init (GtkLayout *layout)
@@ -558,7 +787,6 @@ gtk_layout_realize (GtkWidget *widget)
   GdkWindowAttr attributes;
   gint attributes_mask;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_LAYOUT (widget));
 
   layout = GTK_LAYOUT (widget);
@@ -611,7 +839,6 @@ gtk_layout_map (GtkWidget *widget)
   GList *tmp_list;
   GtkLayout *layout;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_LAYOUT (widget));
 
   layout = GTK_LAYOUT (widget);
@@ -640,7 +867,6 @@ gtk_layout_unrealize (GtkWidget *widget)
 {
   GtkLayout *layout;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_LAYOUT (widget));
 
   layout = GTK_LAYOUT (widget);
@@ -660,7 +886,6 @@ gtk_layout_size_request (GtkWidget     *widget,
   GList *tmp_list;
   GtkLayout *layout;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_LAYOUT (widget));
 
   layout = GTK_LAYOUT (widget);
@@ -688,7 +913,6 @@ gtk_layout_size_allocate (GtkWidget     *widget,
   GList *tmp_list;
   GtkLayout *layout;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_LAYOUT (widget));
 
   widget->allocation = *allocation;
@@ -734,7 +958,6 @@ gtk_layout_expose (GtkWidget *widget, GdkEventExpose *event)
 {
   GtkLayout *layout;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_LAYOUT (widget), FALSE);
 
   layout = GTK_LAYOUT (widget);
@@ -757,7 +980,6 @@ gtk_layout_remove (GtkContainer *container,
   GtkLayout *layout;
   GtkLayoutChild *child = NULL;
   
-  g_return_if_fail (container != NULL);
   g_return_if_fail (GTK_IS_LAYOUT (container));
   
   layout = GTK_LAYOUT (container);
@@ -791,7 +1013,6 @@ gtk_layout_forall (GtkContainer *container,
   GtkLayoutChild *child;
   GList *tmp_list;
 
-  g_return_if_fail (container != NULL);
   g_return_if_fail (GTK_IS_LAYOUT (container));
   g_return_if_fail (callback != NULL);
 
