@@ -227,9 +227,10 @@ check_setugid (void)
 static gchar **
 get_module_path (void)
 {
-  gchar *module_path = g_getenv ("GTK_MODULE_PATH");
-  gchar *exe_prefix = g_getenv("GTK_EXE_PREFIX");
+  const gchar *module_path_env = g_getenv ("GTK_MODULE_PATH");
+  const gchar *exe_prefix = g_getenv("GTK_EXE_PREFIX");
   gchar **result;
+  gchar *module_path;
   gchar *default_dir;
 
   if (exe_prefix)
@@ -242,8 +243,8 @@ get_module_path (void)
       default_dir = g_build_filename (get_gtk_win32_directory (""), "modules", NULL);
 #endif
     }
-  module_path = g_strconcat (module_path ? module_path : "",
-			     module_path ? G_SEARCHPATH_SEPARATOR_S : "",
+  module_path = g_strconcat (module_path_env ? module_path_env : "",
+			     module_path_env ? G_SEARCHPATH_SEPARATOR_S : "",
 			     default_dir, NULL);
 
   result = pango_split_file_list (module_path);
@@ -363,6 +364,28 @@ load_modules (const char *module_str)
   return gtk_modules;
 }
 
+static gboolean do_setlocale = TRUE;
+
+/**
+ * gtk_disable_setlocale:
+ * 
+ * Prevents gtk_init() and gtk_init_check() from automatically
+ * calling setlocale (LC_ALL, ""). You would want to use this
+ * function if you wanted to set the locale for your program
+ * to something other than the user's locale, or if you wanted
+ * to set different values for different locale categories.
+ *
+ * Most programs should not need to call this function.
+ **/
+void
+gtk_disable_setlocale (void)
+{
+  if (gtk_initialized)
+    g_warning ("gtk_disable_setlocale() must be called before gtk_init()");
+    
+  do_setlocale = FALSE;
+}
+
 gboolean
 gtk_init_check (int	 *argc,
 		char   ***argv)
@@ -384,6 +407,9 @@ gtk_init_check (int	 *argc,
   g_set_message_handler (gtk_message);
   g_set_print_handler (gtk_print);
 #endif
+
+  if (do_setlocale)
+    setlocale (LC_ALL, "");
   
   /* Initialize "gdk". We pass along the 'argc' and 'argv'
    *  parameters as they contain information that GDK uses
@@ -589,7 +615,7 @@ gtk_init_check (int	 *argc,
   g_message (""              "YOU ARE USING THE DEVEL BRANCH 1.3.x OF GTK+ WHICH IS CURRENTLY\n"
 	     "                UNDER HEAVY DEVELOPMENT AND FREQUENTLY INTRODUCES INSTABILITIES.\n"
 	     "                if you don't know why you are getting this, you probably want to\n"
-	     "                use the stable branch which can be retrived from\n"
+	     "                use the stable branch which can be retrieved from\n"
 	     "                ftp://ftp.gtk.org/pub/gtk/v1.2/ or via CVS with\n"
 	     "                cvs checkout -r glib-1-2 glib; cvs checkout -r gtk-1-2 gtk+");
 #endif
@@ -656,17 +682,23 @@ gtk_exit (gint errorcode)
 /**
  * gtk_set_locale:
  *
+ * Initializes internationalization support for GTK+. gtk_init()
+ * automatically does this, so there is typically no point
+ * in calling this function.
  *
- * Initializes internationalization support for GTK+.  You
- * should call this function before gtk_init() if your application
- * supports internationalization.
+ * If you are calling this function because you changed the locale
+ * after GTK+ is was initialized, then calling this function
+ * may help a bit. (Note, however, that changing the locale
+ * after GTK+ is initialized may produce inconsistent results and
+ * is not really supported.)
  * 
- *  (In gory detail - sets the current locale according to the
+ * In detail - sets the current locale according to the
  * program environment. This is the same as calling the libc function
  * setlocale (LC_ALL, "") but also takes care of the locale specific
- * setup of the windowing system used by GDK.)
+ * setup of the windowing system used by GDK.
  * 
- * Return value: a string corresponding to the locale set, as with the C library function setlocale()
+ * Return value: a string corresponding to the locale set, as with the
+ * C library function setlocale()
  **/
 gchar*
 gtk_set_locale (void)
