@@ -150,7 +150,15 @@ extern void gtk_marshal_VOID__BOXED_BOXED (GClosure     *closure,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
 
-/* VOID:BOXED,BOXED,POINTER (gtkmarshal.list:46) */
+/* VOID:BOXED,BOXED,BOXED (gtkmarshal.list:46) */
+extern void gtk_marshal_VOID__BOXED_BOXED_BOXED (GClosure     *closure,
+                                                 GValue       *return_value,
+                                                 guint         n_param_values,
+                                                 const GValue *param_values,
+                                                 gpointer      invocation_hint,
+                                                 gpointer      marshal_data);
+
+/* VOID:BOXED,BOXED,POINTER (gtkmarshal.list:47) */
 extern void gtk_marshal_VOID__BOXED_BOXED_POINTER (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -158,7 +166,7 @@ extern void gtk_marshal_VOID__BOXED_BOXED_POINTER (GClosure     *closure,
                                                    gpointer      invocation_hint,
                                                    gpointer      marshal_data);
 
-/* VOID:BOXED,POINTER (gtkmarshal.list:47) */
+/* VOID:BOXED,POINTER (gtkmarshal.list:48) */
 extern void gtk_marshal_VOID__BOXED_POINTER (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -166,7 +174,7 @@ extern void gtk_marshal_VOID__BOXED_POINTER (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:BOXED,OBJECT (gtkmarshal.list:48) */
+/* VOID:BOXED,OBJECT (gtkmarshal.list:49) */
 extern void gtk_marshal_VOID__BOXED_OBJECT (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -174,7 +182,7 @@ extern void gtk_marshal_VOID__BOXED_OBJECT (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:BOXED,STRING,INT (gtkmarshal.list:49) */
+/* VOID:BOXED,STRING,INT (gtkmarshal.list:50) */
 extern void gtk_marshal_VOID__BOXED_STRING_INT (GClosure     *closure,
                                                 GValue       *return_value,
                                                 guint         n_param_values,
@@ -182,7 +190,15 @@ extern void gtk_marshal_VOID__BOXED_STRING_INT (GClosure     *closure,
                                                 gpointer      invocation_hint,
                                                 gpointer      marshal_data);
 
-/* VOID:BOXED,UINT (gtkmarshal.list:50) */
+/* VOID:BOXED,INT (gtkmarshal.list:51) */
+extern void gtk_marshal_VOID__BOXED_INT (GClosure     *closure,
+                                         GValue       *return_value,
+                                         guint         n_param_values,
+                                         const GValue *param_values,
+                                         gpointer      invocation_hint,
+                                         gpointer      marshal_data);
+
+/* VOID:BOXED,UINT (gtkmarshal.list:52) */
 extern void gtk_marshal_VOID__BOXED_UINT (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -190,7 +206,7 @@ extern void gtk_marshal_VOID__BOXED_UINT (GClosure     *closure,
                                           gpointer      invocation_hint,
                                           gpointer      marshal_data);
 
-/* VOID:BOXED,UINT,FLAGS (gtkmarshal.list:51) */
+/* VOID:BOXED,UINT,FLAGS (gtkmarshal.list:53) */
 extern void gtk_marshal_VOID__BOXED_UINT_FLAGS (GClosure     *closure,
                                                 GValue       *return_value,
                                                 guint         n_param_values,
@@ -198,7 +214,7 @@ extern void gtk_marshal_VOID__BOXED_UINT_FLAGS (GClosure     *closure,
                                                 gpointer      invocation_hint,
                                                 gpointer      marshal_data);
 
-/* VOID:BOXED,UINT,UINT (gtkmarshal.list:52) */
+/* VOID:BOXED,UINT,UINT (gtkmarshal.list:54) */
 extern void gtk_marshal_VOID__BOXED_UINT_UINT (GClosure     *closure,
                                                GValue       *return_value,
                                                guint         n_param_values,
@@ -206,9 +222,9 @@ extern void gtk_marshal_VOID__BOXED_UINT_UINT (GClosure     *closure,
                                                gpointer      invocation_hint,
                                                gpointer      marshal_data);
 
-/* VOID:ENUM (gtkmarshal.list:53) */
+/* VOID:ENUM (gtkmarshal.list:55) */
 
-/* VOID:ENUM,ENUM (gtkmarshal.list:54) */
+/* VOID:ENUM,ENUM (gtkmarshal.list:56) */
 extern void gtk_marshal_VOID__ENUM_ENUM (GClosure     *closure,
                                          GValue       *return_value,
                                          guint         n_param_values,
@@ -216,7 +232,7 @@ extern void gtk_marshal_VOID__ENUM_ENUM (GClosure     *closure,
                                          gpointer      invocation_hint,
                                          gpointer      marshal_data);
 
-/* VOID:ENUM,FLOAT (gtkmarshal.list:55) */
+/* VOID:ENUM,FLOAT (gtkmarshal.list:57) */
 extern void gtk_marshal_VOID__ENUM_FLOAT (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -224,7 +240,7 @@ extern void gtk_marshal_VOID__ENUM_FLOAT (GClosure     *closure,
                                           gpointer      invocation_hint,
                                           gpointer      marshal_data);
 
-/* VOID:ENUM,FLOAT,BOOL (gtkmarshal.list:56) */
+/* VOID:ENUM,FLOAT,BOOL (gtkmarshal.list:58) */
 extern void gtk_marshal_VOID__ENUM_FLOAT_BOOLEAN (GClosure     *closure,
                                                   GValue       *return_value,
                                                   guint         n_param_values,
@@ -233,7 +249,7 @@ extern void gtk_marshal_VOID__ENUM_FLOAT_BOOLEAN (GClosure     *closure,
                                                   gpointer      marshal_data);
 #define gtk_marshal_VOID__ENUM_FLOAT_BOOL	gtk_marshal_VOID__ENUM_FLOAT_BOOLEAN
 
-/* VOID:ENUM,INT (gtkmarshal.list:57) */
+/* VOID:ENUM,INT (gtkmarshal.list:59) */
 extern void gtk_marshal_VOID__ENUM_INT (GClosure     *closure,
                                         GValue       *return_value,
                                         guint         n_param_values,
@@ -241,7 +257,7 @@ extern void gtk_marshal_VOID__ENUM_INT (GClosure     *closure,
                                         gpointer      invocation_hint,
                                         gpointer      marshal_data);
 
-/* VOID:ENUM,INT,BOOLEAN (gtkmarshal.list:58) */
+/* VOID:ENUM,INT,BOOLEAN (gtkmarshal.list:60) */
 extern void gtk_marshal_VOID__ENUM_INT_BOOLEAN (GClosure     *closure,
                                                 GValue       *return_value,
                                                 guint         n_param_values,
@@ -249,11 +265,11 @@ extern void gtk_marshal_VOID__ENUM_INT_BOOLEAN (GClosure     *closure,
                                                 gpointer      invocation_hint,
                                                 gpointer      marshal_data);
 
-/* VOID:INT (gtkmarshal.list:59) */
+/* VOID:INT (gtkmarshal.list:61) */
 
-/* VOID:INT,INT (gtkmarshal.list:60) */
+/* VOID:INT,INT (gtkmarshal.list:62) */
 
-/* VOID:INT,INT,BOXED (gtkmarshal.list:61) */
+/* VOID:INT,INT,BOXED (gtkmarshal.list:63) */
 extern void gtk_marshal_VOID__INT_INT_BOXED (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -261,7 +277,7 @@ extern void gtk_marshal_VOID__INT_INT_BOXED (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:INT,INT,INT (gtkmarshal.list:62) */
+/* VOID:INT,INT,INT (gtkmarshal.list:64) */
 extern void gtk_marshal_VOID__INT_INT_INT (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -269,10 +285,10 @@ extern void gtk_marshal_VOID__INT_INT_INT (GClosure     *closure,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
 
-/* VOID:OBJECT (gtkmarshal.list:63) */
+/* VOID:OBJECT (gtkmarshal.list:65) */
 #define gtk_marshal_VOID__OBJECT	g_cclosure_marshal_VOID__OBJECT
 
-/* VOID:OBJECT,BOOLEAN (gtkmarshal.list:64) */
+/* VOID:OBJECT,BOOLEAN (gtkmarshal.list:66) */
 extern void gtk_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -280,7 +296,7 @@ extern void gtk_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                               gpointer      invocation_hint,
                                               gpointer      marshal_data);
 
-/* VOID:OBJECT,BOXED,BOXED (gtkmarshal.list:65) */
+/* VOID:OBJECT,BOXED,BOXED (gtkmarshal.list:67) */
 extern void gtk_marshal_VOID__OBJECT_BOXED_BOXED (GClosure     *closure,
                                                   GValue       *return_value,
                                                   guint         n_param_values,
@@ -288,7 +304,7 @@ extern void gtk_marshal_VOID__OBJECT_BOXED_BOXED (GClosure     *closure,
                                                   gpointer      invocation_hint,
                                                   gpointer      marshal_data);
 
-/* VOID:OBJECT,BOXED,UINT,UINT (gtkmarshal.list:66) */
+/* VOID:OBJECT,BOXED,UINT,UINT (gtkmarshal.list:68) */
 extern void gtk_marshal_VOID__OBJECT_BOXED_UINT_UINT (GClosure     *closure,
                                                       GValue       *return_value,
                                                       guint         n_param_values,
@@ -296,7 +312,7 @@ extern void gtk_marshal_VOID__OBJECT_BOXED_UINT_UINT (GClosure     *closure,
                                                       gpointer      invocation_hint,
                                                       gpointer      marshal_data);
 
-/* VOID:OBJECT,INT,INT (gtkmarshal.list:67) */
+/* VOID:OBJECT,INT,INT (gtkmarshal.list:69) */
 extern void gtk_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -304,7 +320,7 @@ extern void gtk_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
                                               gpointer      invocation_hint,
                                               gpointer      marshal_data);
 
-/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (gtkmarshal.list:68) */
+/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (gtkmarshal.list:70) */
 extern void gtk_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *closure,
                                                               GValue       *return_value,
                                                               guint         n_param_values,
@@ -312,7 +328,7 @@ extern void gtk_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *clos
                                                               gpointer      invocation_hint,
                                                               gpointer      marshal_data);
 
-/* VOID:OBJECT,OBJECT (gtkmarshal.list:69) */
+/* VOID:OBJECT,OBJECT (gtkmarshal.list:71) */
 extern void gtk_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -320,7 +336,7 @@ extern void gtk_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:OBJECT,STRING,STRING (gtkmarshal.list:70) */
+/* VOID:OBJECT,STRING,STRING (gtkmarshal.list:72) */
 extern void gtk_marshal_VOID__OBJECT_STRING_STRING (GClosure     *closure,
                                                     GValue       *return_value,
                                                     guint         n_param_values,
@@ -328,7 +344,7 @@ extern void gtk_marshal_VOID__OBJECT_STRING_STRING (GClosure     *closure,
                                                     gpointer      invocation_hint,
                                                     gpointer      marshal_data);
 
-/* VOID:OBJECT,UINT (gtkmarshal.list:71) */
+/* VOID:OBJECT,UINT (gtkmarshal.list:73) */
 extern void gtk_marshal_VOID__OBJECT_UINT (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -336,7 +352,7 @@ extern void gtk_marshal_VOID__OBJECT_UINT (GClosure     *closure,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
 
-/* VOID:OBJECT,UINT,FLAGS (gtkmarshal.list:72) */
+/* VOID:OBJECT,UINT,FLAGS (gtkmarshal.list:74) */
 extern void gtk_marshal_VOID__OBJECT_UINT_FLAGS (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -344,9 +360,9 @@ extern void gtk_marshal_VOID__OBJECT_UINT_FLAGS (GClosure     *closure,
                                                  gpointer      invocation_hint,
                                                  gpointer      marshal_data);
 
-/* VOID:POINTER (gtkmarshal.list:73) */
+/* VOID:POINTER (gtkmarshal.list:75) */
 
-/* VOID:POINTER,INT (gtkmarshal.list:74) */
+/* VOID:POINTER,INT (gtkmarshal.list:76) */
 extern void gtk_marshal_VOID__POINTER_INT (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -354,7 +370,7 @@ extern void gtk_marshal_VOID__POINTER_INT (GClosure     *closure,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
 
-/* VOID:POINTER,POINTER,POINTER (gtkmarshal.list:75) */
+/* VOID:POINTER,POINTER,POINTER (gtkmarshal.list:77) */
 extern void gtk_marshal_VOID__POINTER_POINTER_POINTER (GClosure     *closure,
                                                        GValue       *return_value,
                                                        guint         n_param_values,
@@ -362,7 +378,7 @@ extern void gtk_marshal_VOID__POINTER_POINTER_POINTER (GClosure     *closure,
                                                        gpointer      invocation_hint,
                                                        gpointer      marshal_data);
 
-/* VOID:POINTER,UINT (gtkmarshal.list:76) */
+/* VOID:POINTER,UINT (gtkmarshal.list:78) */
 extern void gtk_marshal_VOID__POINTER_UINT (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -370,12 +386,20 @@ extern void gtk_marshal_VOID__POINTER_UINT (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:STRING (gtkmarshal.list:77) */
+/* VOID:STRING (gtkmarshal.list:79) */
 #define gtk_marshal_VOID__STRING	g_cclosure_marshal_VOID__STRING
 
-/* VOID:STRING,INT,POINTER (gtkmarshal.list:78) */
+/* VOID:STRING,STRING (gtkmarshal.list:80) */
+extern void gtk_marshal_VOID__STRING_STRING (GClosure     *closure,
+                                             GValue       *return_value,
+                                             guint         n_param_values,
+                                             const GValue *param_values,
+                                             gpointer      invocation_hint,
+                                             gpointer      marshal_data);
 
-/* VOID:UINT,BOXED,UINT,FLAGS,FLAGS (gtkmarshal.list:79) */
+/* VOID:STRING,INT,POINTER (gtkmarshal.list:81) */
+
+/* VOID:UINT,BOXED,UINT,FLAGS,FLAGS (gtkmarshal.list:82) */
 extern void gtk_marshal_VOID__UINT_BOXED_UINT_FLAGS_FLAGS (GClosure     *closure,
                                                            GValue       *return_value,
                                                            guint         n_param_values,
@@ -383,7 +407,7 @@ extern void gtk_marshal_VOID__UINT_BOXED_UINT_FLAGS_FLAGS (GClosure     *closure
                                                            gpointer      invocation_hint,
                                                            gpointer      marshal_data);
 
-/* VOID:UINT,OBJECT,UINT,FLAGS,FLAGS (gtkmarshal.list:80) */
+/* VOID:UINT,OBJECT,UINT,FLAGS,FLAGS (gtkmarshal.list:83) */
 extern void gtk_marshal_VOID__UINT_OBJECT_UINT_FLAGS_FLAGS (GClosure     *closure,
                                                             GValue       *return_value,
                                                             guint         n_param_values,
@@ -391,7 +415,7 @@ extern void gtk_marshal_VOID__UINT_OBJECT_UINT_FLAGS_FLAGS (GClosure     *closur
                                                             gpointer      invocation_hint,
                                                             gpointer      marshal_data);
 
-/* VOID:UINT,STRING (gtkmarshal.list:81) */
+/* VOID:UINT,STRING (gtkmarshal.list:84) */
 extern void gtk_marshal_VOID__UINT_STRING (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -399,7 +423,7 @@ extern void gtk_marshal_VOID__UINT_STRING (GClosure     *closure,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
 
-/* VOID:VOID (gtkmarshal.list:82) */
+/* VOID:VOID (gtkmarshal.list:85) */
 
 G_END_DECLS
 
