@@ -134,7 +134,7 @@ gtk_arg_set_from_value (GtkArg       *arg,
     case G_TYPE_STRING:         if (copy_string)
       GTK_VALUE_STRING (*arg) = g_value_dup_string (value);
     else
-      GTK_VALUE_STRING (*arg) = g_value_get_string (value);
+      GTK_VALUE_STRING (*arg) = (char *) g_value_get_string (value);
     break;
     default:
       return FALSE;
@@ -354,12 +354,7 @@ gtk_object_destroy (GtkObject *object)
   g_return_if_fail (GTK_IS_OBJECT (object));
   
   if (!GTK_OBJECT_DESTROYED (object))
-    {
-      /* need to hold a reference count around all class method
-       * invocations.
-       */
       g_object_run_dispose (G_OBJECT (object));
-    }
 }
 
 static void
