@@ -129,6 +129,11 @@ gboolean gtk_selection_data_set_text (GtkSelectionData     *selection_data,
 				      const guchar         *str);
 guchar * gtk_selection_data_get_text (GtkSelectionData     *selection_data);
 
+gboolean gtk_selection_data_get_targets          (GtkSelectionData  *selection_data,
+						  GdkAtom          **targets,
+						  gint              *n_atoms);
+gboolean gtk_selection_data_targets_include_text (GtkSelectionData  *selection_data);
+
 /* Called when a widget is destroyed */
 
 void gtk_selection_remove_all      (GtkWidget *widget);
