@@ -40,7 +40,6 @@
 #include "gtkintl.h"
 
 #define MIN_SPIN_BUTTON_WIDTH              30
-#define ARROW_SIZE                         11
 #define SPIN_BUTTON_INITIAL_TIMER_DELAY    200
 #define SPIN_BUTTON_TIMER_DELAY            20
 #define MAX_TIMER_CALLS                    5
@@ -69,6 +68,7 @@ enum
 };
 
 static void gtk_spin_button_class_init     (GtkSpinButtonClass *klass);
+static void gtk_spin_button_editable_init  (GtkEditableClass   *iface);
 static void gtk_spin_button_init           (GtkSpinButton      *spin_button);
 static void gtk_spin_button_finalize       (GObject            *object);
 static void gtk_spin_button_set_property   (GObject         *object,
@@ -115,7 +115,7 @@ static gint gtk_spin_button_scroll         (GtkWidget          *widget,
 static void gtk_spin_button_activate       (GtkEntry           *entry);
 static void gtk_spin_button_snap           (GtkSpinButton      *spin_button,
 					    gdouble             val);
-static void gtk_spin_button_insert_text    (GtkEntry           *entry,
+static void gtk_spin_button_insert_text    (GtkEditable        *editable,
 					    const gchar        *new_text,
 					    gint                new_text_length,
 					    gint               *position);
@@ -124,6 +124,7 @@ static void gtk_spin_button_real_spin      (GtkSpinButton      *spin_button,
 static gint gtk_spin_button_default_input  (GtkSpinButton      *spin_button,
 					    gdouble            *new_val);
 static gint gtk_spin_button_default_output (GtkSpinButton      *spin_button);
+static gint spin_button_get_arrow_size     (GtkSpinButton      *spin_button);
 static gint spin_button_get_shadow_type    (GtkSpinButton      *spin_button);
 
 
@@ -150,7 +151,17 @@ gtk_spin_button_get_type (void)
         (GtkClassInitFunc) NULL,
       };
 
+      static const GInterfaceInfo editable_info =
+      {
+	(GInterfaceInitFunc) gtk_spin_button_editable_init, /* interface_init */
+	NULL, /* interface_finalize */
+	NULL  /* interface_data */
+      };
+
       spin_button_type = gtk_type_unique (GTK_TYPE_ENTRY, &spin_button_info);
+      g_type_add_interface_static (spin_button_type,
+				   GTK_TYPE_EDITABLE,
+				   &editable_info);
     }
   return spin_button_type;
 }
@@ -165,7 +176,7 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
 
   object_class   = (GtkObjectClass*)   class;
   widget_class   = (GtkWidgetClass*)   class;
-  entry_class    = (GtkEntryClass*)    class; 
+  entry_class    = (GtkEntryClass*)    class;
 
   parent_class = gtk_type_class (GTK_TYPE_ENTRY);
 
@@ -191,7 +202,6 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
   widget_class->leave_notify_event = gtk_spin_button_leave_notify;
   widget_class->focus_out_event = gtk_spin_button_focus_out;
 
-  entry_class->insert_text = gtk_spin_button_insert_text;
   entry_class->activate = gtk_spin_button_activate;
 
   class->input = NULL;
@@ -300,6 +310,12 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
 		    GTK_TYPE_NONE, 0);
 }
 
+static void
+gtk_spin_button_editable_init (GtkEditableClass *iface)
+{
+  iface->insert_text = gtk_spin_button_insert_text;
+}
+
 static void
 gtk_spin_button_set_property (GObject      *object,
 			      guint         prop_id,
@@ -459,13 +475,15 @@ gtk_spin_button_realize (GtkWidget *widget)
   gint attributes_mask;
   guint real_width;
   gint return_val;
+  gint arrow_size;
 
   g_return_if_fail (GTK_IS_SPIN_BUTTON (widget));
   
   spin_button = GTK_SPIN_BUTTON (widget);
+  arrow_size = spin_button_get_arrow_size (spin_button);
 
   real_width = widget->allocation.width;
-  widget->allocation.width -= ARROW_SIZE + 2 * widget->style->xthickness;
+  widget->allocation.width -= arrow_size + 2 * widget->style->xthickness;
   gtk_widget_set_events (widget, gtk_widget_get_events (widget) |
 			 GDK_KEY_RELEASE_MASK);
   GTK_WIDGET_CLASS (parent_class)->realize (widget);
@@ -483,11 +501,12 @@ gtk_spin_button_realize (GtkWidget *widget)
 
   attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
 
-  attributes.x = (widget->allocation.x + widget->allocation.width - ARROW_SIZE -
+  attributes.x = (widget->allocation.x +
+		  widget->allocation.width - arrow_size -
 		  2 * widget->style->xthickness);
   attributes.y = widget->allocation.y + (widget->allocation.height -
 					 widget->requisition.height) / 2;
-  attributes.width = ARROW_SIZE + 2 * widget->style->xthickness;
+  attributes.width = arrow_size + 2 * widget->style->xthickness;
   attributes.height = widget->requisition.height;
   
   spin_button->panel = gdk_window_new (gtk_widget_get_parent_window (widget), 
@@ -551,12 +570,14 @@ gtk_spin_button_size_request (GtkWidget      *widget,
 {
   GtkEntry *entry;
   GtkSpinButton *spin_button;
+  gint arrow_size;
   
   g_return_if_fail (requisition != NULL);
   g_return_if_fail (GTK_IS_SPIN_BUTTON (widget));
 
   entry = GTK_ENTRY (widget);
   spin_button = GTK_SPIN_BUTTON (widget);
+  arrow_size = spin_button_get_arrow_size (spin_button);
   
   GTK_WIDGET_CLASS (parent_class)->size_request (widget, requisition);
 
@@ -567,6 +588,7 @@ gtk_spin_button_size_request (GtkWidget      *widget,
       gint width;
       gint w;
       gint string_len;
+      gint max_string_len;
       gint digit_width;
 
       context = gtk_widget_get_pango_context (widget);
@@ -582,37 +604,45 @@ gtk_spin_button_size_request (GtkWidget      *widget,
       /* Get max of MIN_SPIN_BUTTON_WIDTH, size of upper, size of lower */
       
       width = MIN_SPIN_BUTTON_WIDTH;
+      max_string_len = MAX (10, compute_double_length (1e9 * spin_button->adjustment->step_increment,
+                                                       spin_button->digits));
 
       string_len = compute_double_length (spin_button->adjustment->upper,
                                           spin_button->digits);
-      w = MIN (string_len, 10) * digit_width;
+      w = MIN (string_len, max_string_len) * digit_width;
       width = MAX (width, w);
       string_len = compute_double_length (spin_button->adjustment->lower,
 					  spin_button->digits);
-      w = MIN (string_len, 10) * digit_width;
+      w = MIN (string_len, max_string_len) * digit_width;
       width = MAX (width, w);
       
-      requisition->width = width + ARROW_SIZE + 2 * widget->style->xthickness;
+      requisition->width = (width + arrow_size +
+			    2 * widget->style->xthickness);
     }
   else
-    requisition->width += ARROW_SIZE + 2 * widget->style->xthickness;
+    requisition->width += arrow_size + 2 * widget->style->xthickness;
 }
 
 static void
 gtk_spin_button_size_allocate (GtkWidget     *widget,
 			       GtkAllocation *allocation)
 {
+  GtkSpinButton *spin;
   GtkAllocation child_allocation;
+  gint arrow_size;
 
   g_return_if_fail (GTK_IS_SPIN_BUTTON (widget));
   g_return_if_fail (allocation != NULL);
 
+  spin = GTK_SPIN_BUTTON (widget);
+  arrow_size = spin_button_get_arrow_size (spin);
+
   child_allocation = *allocation;
-  if (child_allocation.width > ARROW_SIZE + 2 * widget->style->xthickness)
-    child_allocation.width -= ARROW_SIZE + 2 * widget->style->xthickness;
+  if (child_allocation.width > arrow_size + 2 * widget->style->xthickness)
+    child_allocation.width -= arrow_size + 2 * widget->style->xthickness;
 
   if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
-    child_allocation.x += ARROW_SIZE + 2 * widget->style->xthickness;
+    child_allocation.x += arrow_size + 2 * widget->style->xthickness;
 
   GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, &child_allocation);
 
@@ -620,12 +650,12 @@ gtk_spin_button_size_allocate (GtkWidget     *widget,
 
   if (GTK_WIDGET_REALIZED (widget))
     {
-      child_allocation.width = ARROW_SIZE + 2 * widget->style->xthickness;
+      child_allocation.width = arrow_size + 2 * widget->style->xthickness;
       child_allocation.height = widget->requisition.height;
 
       if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR)
-	child_allocation.x = (allocation->x + allocation->width - ARROW_SIZE - 
-			      2 * widget->style->xthickness);
+	child_allocation.x = (allocation->x + allocation->width -
+			      arrow_size - 2 * widget->style->xthickness);
       else
 	child_allocation.x = allocation->x;      
 
@@ -644,11 +674,13 @@ gtk_spin_button_expose (GtkWidget      *widget,
 			GdkEventExpose *event)
 {
   GtkSpinButton *spin;
+  gint arrow_size;
 
   g_return_val_if_fail (GTK_IS_SPIN_BUTTON (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
   spin = GTK_SPIN_BUTTON (widget);
+  arrow_size = spin_button_get_arrow_size (spin);
 
   if (GTK_WIDGET_DRAWABLE (widget))
     {
@@ -665,7 +697,7 @@ gtk_spin_button_expose (GtkWidget      *widget,
 		       GTK_STATE_NORMAL, shadow_type,
 		       &event->area, widget, "spinbutton",
 		       0, 0, 
-		       ARROW_SIZE + 2 * widget->style->xthickness,
+		       arrow_size + 2 * widget->style->xthickness,
 		       widget->requisition.height); 
       else
 	 {
@@ -693,11 +725,13 @@ gtk_spin_button_draw_arrow (GtkSpinButton *spin_button,
   GtkWidget *widget;
   gint x;
   gint y;
+  gint arrow_size;
 
   g_return_if_fail (GTK_IS_SPIN_BUTTON (spin_button));
   
   widget = GTK_WIDGET (spin_button);
   spin_shadow_type = spin_button_get_shadow_type (spin_button);
+  arrow_size = spin_button_get_arrow_size (spin_button);
 
   if (GTK_WIDGET_DRAWABLE (spin_button))
     {
@@ -745,7 +779,8 @@ gtk_spin_button_draw_arrow (GtkSpinButton *spin_button,
 			   state_type, shadow_type, 
 			   NULL, widget, "spinbutton",
 			   arrow, TRUE, 
-			   x, y, ARROW_SIZE, widget->requisition.height / 2 
+			   x, y, arrow_size,
+			   widget->requisition.height / 2 
 			   - widget->style->ythickness);
 	}
       else
@@ -764,7 +799,8 @@ gtk_spin_button_draw_arrow (GtkSpinButton *spin_button,
 			   state_type, shadow_type, 
 			   NULL, widget, "spinbutton",
 			   arrow, TRUE, 
-			   x, y, ARROW_SIZE, widget->requisition.height / 2 
+			   x, y, arrow_size,
+			   widget->requisition.height / 2 
 			   - widget->style->ythickness);
 	}
     }
@@ -971,11 +1007,13 @@ gtk_spin_button_button_release (GtkWidget      *widget,
 				GdkEventButton *event)
 {
   GtkSpinButton *spin;
+  gint arrow_size;
 
   g_return_val_if_fail (GTK_IS_SPIN_BUTTON (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
   spin = GTK_SPIN_BUTTON (widget);
+  arrow_size = spin_button_get_arrow_size (spin);
 
   if (event->button == spin->button)
     {
@@ -993,7 +1031,7 @@ gtk_spin_button_button_release (GtkWidget      *widget,
 	{
 	  if (event->y >= 0 && event->x >= 0 && 
 	      event->y <= widget->requisition.height &&
-	      event->x <= ARROW_SIZE + 2 * widget->style->xthickness)
+	      event->x <= arrow_size + 2 * widget->style->xthickness)
 	    {
 	      if (spin->click_child == GTK_ARROW_UP &&
 		  event->y <= widget->requisition.height / 2)
@@ -1309,13 +1347,14 @@ gtk_spin_button_activate (GtkEntry *entry)
 }
 
 static void
-gtk_spin_button_insert_text (GtkEntry    *entry,
+gtk_spin_button_insert_text (GtkEditable *editable,
 			     const gchar *new_text,
 			     gint         new_text_length,
 			     gint        *position)
 {
-  GtkEditable *editable = GTK_EDITABLE (entry);
+  GtkEntry *entry = GTK_ENTRY (editable);
   GtkSpinButton *spin = GTK_SPIN_BUTTON (editable);
+  GtkEditableClass *parent_editable_iface = g_type_interface_peek (parent_class, GTK_TYPE_EDITABLE);
  
   if (spin->numeric)
     {
@@ -1385,8 +1424,8 @@ gtk_spin_button_insert_text (GtkEntry    *entry,
 	}
     }
 
-  GTK_ENTRY_CLASS (parent_class)->insert_text (entry, new_text,
-					       new_text_length, position);
+  parent_editable_iface->insert_text (editable, new_text,
+				      new_text_length, position);
 }
 
 static void
@@ -1933,6 +1972,14 @@ gtk_spin_button_get_wrap (GtkSpinButton *spin_button)
   return spin_button->wrap;
 }
 
+static gint
+spin_button_get_arrow_size (GtkSpinButton *spin_button)
+{
+  gint size = pango_font_description_get_size (GTK_WIDGET (spin_button)->style->font_desc);
+  
+  return PANGO_PIXELS (size);
+}
+
 /**
  * spin_button_get_shadow_type:
  * @spin_button: a #GtkSpinButton 
