@@ -34,6 +34,12 @@
 extern "C" {
 #endif /* __cplusplus */
 
+typedef enum {
+  GTK_TEXT_SEARCH_VISIBLE_ONLY,
+  GTK_TEXT_SEARCH_TEXT_ONLY
+  /* Possible future plans: SEARCH_CASE_INSENSITIVE, SEARCH_REGEXP */
+} GtkTextSearchFlags;
+
 /*
  * Iter: represents a location in the text. Becomes invalid if the
  * characters/pixmaps/widgets (indexable objects) in the text buffer
@@ -43,6 +49,11 @@ extern "C" {
 typedef struct _GtkTextBuffer GtkTextBuffer;
 
 struct _GtkTextIter {
+  /* GtkTextIter is an opaque datatype; ignore all these fields.
+   * Initialize the iter with gtk_text_buffer_get_iter_*
+   * functions
+   */
+  /*< private >*/
   gpointer dummy1;
   gpointer dummy2;
   gint dummy3;
@@ -128,6 +139,8 @@ GSList  *gtk_text_iter_get_tags          (const GtkTextIter   *iter);
 
 gboolean gtk_text_iter_editable          (const GtkTextIter   *iter,
                                           gboolean             default_setting);
+gboolean gtk_text_iter_can_insert        (const GtkTextIter   *iter,
+                                          gboolean             default_editability);
 
 gboolean gtk_text_iter_starts_word        (const GtkTextIter   *iter);
 gboolean gtk_text_iter_ends_word          (const GtkTextIter   *iter);
@@ -229,16 +242,14 @@ gboolean gtk_text_iter_backward_find_char (GtkTextIter          *iter,
 
 gboolean gtk_text_iter_forward_search  (const GtkTextIter *iter,
                                         const gchar       *str,
-                                        gboolean           visible_only,
-                                        gboolean           slice,
+                                        GtkTextSearchFlags flags,
                                         GtkTextIter       *match_start,
                                         GtkTextIter       *match_end,
                                         const GtkTextIter *limit);
 
 gboolean gtk_text_iter_backward_search (const GtkTextIter *iter,
                                         const gchar       *str,
-                                        gboolean           visible_only,
-                                        gboolean           slice,
+                                        GtkTextSearchFlags flags,
                                         GtkTextIter       *match_start,
                                         GtkTextIter       *match_end,
                                         const GtkTextIter *limit);
