@@ -103,6 +103,7 @@ _gtk_tree_data_list_check_type (GType type)
   return FALSE;
 }
 
+
 void
 _gtk_tree_data_list_node_to_value (GtkTreeDataList *list,
 				   GType            type,
@@ -264,7 +265,7 @@ gtk_tree_data_list_compare_func (GtkTreeModel *model,
   GValue a_value = {0, };
   GValue b_value = {0, };
   gint retval;
-  gchar *stra, *strb;
+  const gchar *stra, *strb;
 
   gtk_tree_model_get_value (model, a, column, &a_value);
   gtk_tree_model_get_value (model, b, column, &b_value);
@@ -302,11 +303,9 @@ gtk_tree_data_list_compare_func (GtkTreeModel *model,
     case G_TYPE_STRING:
       stra = g_value_get_string (&a_value);
       strb = g_value_get_string (&b_value);
-      if (stra == NULL)
-	stra = "";
-      if (strb == NULL)
-	strb = "";
-      retval = strcmp (stra, strb);
+      if (stra == NULL) stra = "";
+      if (strb == NULL) strb = "";
+      retval = g_utf8_collate (stra, strb);
       break;
     case G_TYPE_POINTER:
     case G_TYPE_BOXED:
