@@ -651,6 +651,39 @@ gtk_tree_model_get_iter (GtkTreeModel *tree_model,
   return (* GTK_TREE_MODEL_GET_IFACE (tree_model)->get_iter) (tree_model, iter, path);
 }
 
+/**
+ * gtk_tree_model_get_iter_from_string:
+ * @tree_model: A #GtkTreeModel.
+ * @iter: An uninitialized #GtkTreeIter.
+ * @path_string: A string representation of a #GtkTreePath.
+ *
+ * Sets @iter to a valid iterator pointing to @path_string, if it
+ * exists. Otherwise, @iter is left invalid and %FALSE is returned.
+ *
+ * Return value: TRUE, if @iter was set.
+ **/
+gboolean
+gtk_tree_model_get_iter_from_string (GtkTreeModel *tree_model,
+				     GtkTreeIter  *iter,
+				     const gchar  *path_string)
+{
+  gboolean retval;
+  GtkTreePath *path;
+
+  g_return_val_if_fail (GTK_IS_TREE_MODEL (tree_model), FALSE);
+  g_return_val_if_fail (iter != NULL, FALSE);
+  g_return_val_if_fail (path_string != NULL, FALSE);
+  
+  path = gtk_tree_path_new_from_string (path_string);
+  
+  g_return_val_if_fail (path != NULL, FALSE);
+
+  retval = gtk_tree_model_get_iter (tree_model, iter, path);
+  gtk_tree_path_free (path);
+  
+  return retval;
+}
+
 
 /**
  * gtk_tree_model_get_iter_root:
@@ -1146,6 +1179,10 @@ gtk_tree_model_foreach (GtkTreeModel            *model,
  * GtkTreeRowReference
  */
 
+static void gtk_tree_row_reference_unref_path (GtkTreePath  *path,
+					       GtkTreeModel *model,
+					       gboolean      free_last);
+
 #define ROW_REF_DATA_STRING "gtk-tree-row-refs"
 
 struct _GtkTreeRowReference
@@ -1213,25 +1250,29 @@ gtk_tree_row_ref_inserted_callback (GObject     *object,
     {
       GtkTreeRowReference *reference = tmp_list->data;
 
-      if (reference->path)
+      if (reference->path == NULL)
+	goto done;
+
+      if (reference->path->depth >= path->depth)
 	{
-	  gint depth = gtk_tree_path_get_depth (path);
-	  gint ref_depth = gtk_tree_path_get_depth (reference->path);
+	  gint i;
+	  gboolean ancestor = TRUE;
 
-	  if (ref_depth >= depth)
+	  for (i = 0; i < path->depth - 1; i ++)
 	    {
-	      gint *indices = gtk_tree_path_get_indices (path);
-	      gint *ref_indices = gtk_tree_path_get_indices (reference->path);
-	      gint i;
-
-	      /* This is the depth that might affect us. */
-	      i = depth - 1;
-
-	      if (indices[i] <= ref_indices[i])
-		ref_indices[i] += 1;
+	      if (path->indices[i] != reference->path->indices[i])
+		{
+		  ancestor = FALSE;
+		  break;
+		}
 	    }
-	}
+	  if (ancestor == FALSE)
+	    goto done;
 
+	  if (path->indices[path->depth-1] <= reference->path->indices[path->depth-1])
+	    reference->path->indices[path->depth-1] += 1;
+	}
+    done:
       tmp_list = g_slist_next (tmp_list);
     }
 }
@@ -1252,10 +1293,6 @@ gtk_tree_row_ref_deleted_callback (GObject     *object,
    * deletion with the old path of the just-deleted row. Which means
    * that the deleted path is the same now-defunct "coordinate system"
    * as the path saved in the reference, which is what we want to fix.
-   *
-   * Note that this is different from the situation in "inserted," so
-   * while you might think you can cut-and-paste between these
-   * functions, it's not going to work. ;-)
    */
 
   tmp_list = refs->list;
@@ -1266,30 +1303,16 @@ gtk_tree_row_ref_deleted_callback (GObject     *object,
 
       if (reference->path)
 	{
-	  gint depth = gtk_tree_path_get_depth (path);
-	  gint ref_depth = gtk_tree_path_get_depth (reference->path);
-
-	  if (ref_depth >= depth)
+	  if (gtk_tree_path_is_ancestor (path, reference->path))
 	    {
-	      /* Need to adjust path upward */
-	      gint *indices = gtk_tree_path_get_indices (path);
-	      gint *ref_indices = gtk_tree_path_get_indices (reference->path);
-	      gint i;
-
-	      i = depth - 1;
-	      if (indices[i] < ref_indices[i])
-		ref_indices[i] -= 1;
-	      else if (indices[i] == ref_indices[i])
-		{
-		  /* the referenced node itself, or its parent, was
-		   * deleted, mark invalid
-		   */
-
-		  gtk_tree_path_free (reference->path);
-		  reference->path = NULL;
-		}
+	      reference->path->indices[path->depth-1]-=1;
+	    }
+	  else if (gtk_tree_path_compare (path, reference->path) == 0)
+	    {
+	      gtk_tree_row_reference_unref_path (reference->path, reference->model, FALSE);
+	      gtk_tree_path_free (reference->path);
+	      reference->path = NULL;
 	    }
-
 	}
       tmp_list = g_slist_next (tmp_list);
     }
@@ -1364,6 +1387,42 @@ connect_ref_callbacks (GtkTreeModel *model)
 		    model);
 }
 
+
+/* We do this recursively so that we can unref children nodes before their parent */
+static void
+gtk_tree_row_reference_unref_path_helper (GtkTreePath  *path,
+					  GtkTreeModel *model,
+					  GtkTreeIter  *parent_iter,
+					  gint          depth,
+					  gboolean      free_last)
+{
+  GtkTreeIter iter;
+
+  if (free_last == FALSE && path->depth - 1 == depth)
+    return;
+  if (path->depth == depth)
+    return;
+
+  gtk_tree_model_iter_nth_child (model, &iter, parent_iter, path->indices[depth]);
+  gtk_tree_row_reference_unref_path_helper (path, model, &iter, depth + 1, free_last);
+  gtk_tree_model_unref_node (model, &iter);
+}
+
+static void
+gtk_tree_row_reference_unref_path (GtkTreePath  *path,
+				   GtkTreeModel *model,
+				   gboolean      free_last)
+{
+  GtkTreeIter iter;
+
+  if (free_last == FALSE && path->depth == 1)
+    return;
+      
+  gtk_tree_model_iter_nth_child (model, &iter, NULL, path->indices[0]);
+  gtk_tree_row_reference_unref_path_helper (path, model, &iter, 1, free_last);
+  gtk_tree_model_unref_node (model, &iter);
+}
+
 static void
 disconnect_ref_callbacks (GtkTreeModel *model)
 {
@@ -1384,6 +1443,18 @@ disconnect_ref_callbacks (GtkTreeModel *model)
 					NULL);
 }
 
+/**
+ * gtk_tree_row_reference_new:
+ * @model: A #GtkTreeModel
+ * @path: A valid #GtkTreePath
+ * 
+ * Creates a row reference based on @path.  This reference will keep pointing to
+ * the node pointed to by @path, so long as it exists.  It listens to all
+ * signals on model, and updates it's path appropriately.  If @path isn't a
+ * valid path in @model, then %NULL is returned.
+ * 
+ * Return value: A newly allocated #GtkTreeRowReference, or %NULL
+ **/
 GtkTreeRowReference *
 gtk_tree_row_reference_new (GtkTreeModel *model,
                             GtkTreePath  *path)
@@ -1401,11 +1472,31 @@ gtk_tree_row_reference_new_proxy (GObject      *proxy,
 {
   GtkTreeRowReference *reference;
   RowRefList *refs;
+  GtkTreeIter parent_iter;
+  gint i;
 
   g_return_val_if_fail (G_IS_OBJECT (proxy), NULL);
   g_return_val_if_fail (GTK_IS_TREE_MODEL (model), NULL);
   g_return_val_if_fail (path != NULL, NULL);
+  g_return_val_if_fail (path->depth > 0, NULL);
 
+  /* check that the path is valid */
+  if (gtk_tree_model_get_iter (model, &parent_iter, path) == FALSE)
+    return NULL;
+
+  /* Now we want to ref every node */
+  gtk_tree_model_iter_nth_child (model, &parent_iter, NULL, path->indices[0]);
+  gtk_tree_model_ref_node (model, &parent_iter);
+
+  for (i = 1; i < path->depth; i++)
+    {
+      GtkTreeIter iter;
+      gtk_tree_model_iter_nth_child (model, &iter, &parent_iter, path->indices[i]);
+      gtk_tree_model_ref_node (model, &iter);
+      parent_iter = iter;
+    }
+
+  /* Make ther row reference */
   reference = g_new (GtkTreeRowReference, 1);
 
   g_object_ref (proxy);
@@ -1510,7 +1601,10 @@ gtk_tree_row_reference_free (GtkTreeRowReference *reference)
   g_object_unref (reference->model);
 
   if (reference->path)
-    gtk_tree_path_free (reference->path);
+    {
+      gtk_tree_row_reference_unref_path (reference->path, reference->model, TRUE);
+      gtk_tree_path_free (reference->path);
+    }
 
   g_free (reference);
 }
