@@ -89,7 +89,9 @@ struct _GtkTreeViewClass
   void     (* move_cursor)                (GtkTreeView       *tree_view,
 				           GtkMovementStep    step,
 				           gint               count);
-  void     (* select_cursor_row)          (GtkTreeView       *tree_view);
+  void     (* select_all)                 (GtkTreeView       *tree_view);
+  void     (* select_cursor_row)          (GtkTreeView       *tree_view,
+					   gboolean           start_editing);
   void     (* toggle_cursor_row)          (GtkTreeView       *tree_view);
   void     (* expand_collapse_cursor_row) (GtkTreeView       *tree_view,
 					   gboolean           logical,
@@ -119,7 +121,7 @@ typedef gboolean (* GtkTreeViewDroppableFunc)  (GtkTreeView             *tree_vi
 						gpointer                 user_data);
 typedef gboolean (*GtkTreeViewSearchEqualFunc) (GtkTreeModel            *model,
 						gint                     column,
-						gchar                   *key,
+						const gchar             *key,
 						GtkTreeIter             *iter,
 						gpointer                 search_data);
 
@@ -286,15 +288,15 @@ GdkPixmap             *gtk_tree_view_create_row_drag_icon          (GtkTreeView
 
 /* Interactive search */
 void                       gtk_tree_view_set_enable_search     (GtkTreeView                *tree_view,
-								gboolean                    use_search);
+								gboolean                    enable_search);
 gboolean                   gtk_tree_view_get_enable_search     (GtkTreeView                *tree_view);
 gint                       gtk_tree_view_get_search_column     (GtkTreeView                *tree_view);
 void                       gtk_tree_view_set_search_column     (GtkTreeView                *tree_view,
 								gint                        column);
 GtkTreeViewSearchEqualFunc gtk_tree_view_get_search_equal_func (GtkTreeView                *tree_view);
 void                       gtk_tree_view_set_search_equal_func (GtkTreeView                *tree_view,
-								GtkTreeViewSearchEqualFunc  search_compare_func,
-								gpointer                    search_data,
+								GtkTreeViewSearchEqualFunc  search_equal_func,
+								gpointer                    search_user_data,
 								GtkDestroyNotify            search_destroy);
 
 /* This function should really never be used.  It is just for use by ATK.
