@@ -300,7 +300,6 @@ static const GtkEnumValue _gtk_path_priority_type_values[] = {
   { GTK_PATH_PRIO_THEME, "GTK_PATH_PRIO_THEME", "theme" },
   { GTK_PATH_PRIO_RC, "GTK_PATH_PRIO_RC", "rc" },
   { GTK_PATH_PRIO_HIGHEST, "GTK_PATH_PRIO_HIGHEST", "highest" },
-  { GTK_PATH_PRIO_MASK, "GTK_PATH_PRIO_MASK", "mask" },
   { 0, NULL, NULL }
 };
 static const GtkEnumValue _gtk_path_type_values[] = {
@@ -603,6 +602,13 @@ static const GtkEnumValue _gtk_spin_type_values[] = {
   { 0, NULL, NULL }
 };
 
+/* enumerations from "gtktextiter.h" */
+static const GtkEnumValue _gtk_text_search_flags_values[] = {
+  { GTK_TEXT_SEARCH_VISIBLE_ONLY, "GTK_TEXT_SEARCH_VISIBLE_ONLY", "visible-only" },
+  { GTK_TEXT_SEARCH_TEXT_ONLY, "GTK_TEXT_SEARCH_TEXT_ONLY", "text-only" },
+  { 0, NULL, NULL }
+};
+
 /* enumerations from "gtktextview.h" */
 static const GtkEnumValue _gtk_text_window_type_values[] = {
   { GTK_TEXT_WINDOW_PRIVATE, "GTK_TEXT_WINDOW_PRIVATE", "private" },
