@@ -264,7 +264,7 @@ gtk_im_context_xim_new (void)
   if (!info)
     return NULL;
 
-  result = GTK_IM_CONTEXT_XIM (gtk_type_new (GTK_TYPE_IM_CONTEXT_XIM));
+  result = GTK_IM_CONTEXT_XIM (g_object_new (GTK_TYPE_IM_CONTEXT_XIM, NULL));
 
   result->im_info = info;
   
@@ -361,7 +361,7 @@ gtk_im_context_xim_filter_keypress (GtkIMContext *context,
 					       * control characters into strings
 					       */
 	    {
-	      gtk_signal_emit_by_name (GTK_OBJECT (context), "commit", result_utf8);
+	      g_signal_emit_by_name (context, "commit", result_utf8);
 	      result = TRUE;
 	    }
 	  
@@ -468,7 +468,7 @@ gtk_im_context_xim_reset (GtkIMContext *context)
       char *result_utf8 = mb_to_utf8 (context_xim, result);
       if (result_utf8)
 	{
-	  gtk_signal_emit_by_name (GTK_OBJECT (context), "commit", result_utf8);
+	  g_signal_emit_by_name (context, "commit", result_utf8);
 	  g_free (result_utf8);
 	}
     }
@@ -476,7 +476,7 @@ gtk_im_context_xim_reset (GtkIMContext *context)
   if (context_xim->preedit_length)
     {
       context_xim->preedit_length = 0;
-      gtk_signal_emit_by_name (GTK_OBJECT (context), "preedit_changed");
+      g_signal_emit_by_name (context, "preedit_changed");
     }
 
   XFree (result);
@@ -576,7 +576,7 @@ preedit_start_callback (XIC      xic,
 {
   GtkIMContext *context = GTK_IM_CONTEXT (client_data);
   
-  gtk_signal_emit_by_name (GTK_OBJECT (context), "preedit_start");
+  g_signal_emit_by_name (context, "preedit_start");
   g_print ("Starting preedit!\n");
 }		     
 
@@ -587,7 +587,7 @@ preedit_done_callback (XIC      xic,
 {
   GtkIMContext *context = GTK_IM_CONTEXT (client_data);
   
-  gtk_signal_emit_by_name (GTK_OBJECT (context), "preedit_end");  
+  g_signal_emit_by_name (context, "preedit_end");  
   g_print ("Ending preedit!\n");
 }		     
 
@@ -713,7 +713,7 @@ preedit_draw_callback (XIC                           xic,
   if (new_text)
     g_free (new_text);
 
-  gtk_signal_emit_by_name (GTK_OBJECT (context), "preedit_changed");
+  g_signal_emit_by_name (context, "preedit_changed");
 }
     
 
@@ -727,7 +727,7 @@ preedit_caret_callback (XIC                            xic,
   if (call_data->direction == XIMAbsolutePosition)
     {
       context->preedit_cursor = call_data->position;
-      gtk_signal_emit_by_name (GTK_OBJECT (context), "preedit_changed");
+      g_signal_emit_by_name (context, "preedit_changed");
     }
   else
     {
